{-# LANGUAGE MultiParamTypeClasses #-}
{-# LANGUAGE FlexibleContexts #-}

module Framework.ExtCallsCriteria (

-- * Exported data

InfCheckerCriteria (..), Prover9Criteria (..), CritPairInfCheckerInfo (..),

-- * Exported functions

buildInfCheckerCriteria, buildInfCheckerCriteriaCTRS, buildProver9Criteria, getNonRepeatedVar, buildInfCheckerCriteriaRCondsCRule,
termVarsToCons, termSVarsToVars, ruleSVarsToVars, sVarsToVars, cruleSVarsToVars, condSVarsToVars, buildProver9CriteriaCSR,
getJoinTerminationCRules

) where

import Framework.Problems (CriticalPair (..), CriticalPairInfo (..))
import Framework.TRSAux (maxNumOfConds)
import Utils.Auxiliar (replaceNElem)

import TRS.TRSs(HasSignature (..), HasRules (..), HasCRules (..), HasOSSignature(..))
import TRS.Terms(Term(..), vars, fresh)
import TRS.Rules(CRule (..), Rule (..), Cond (..), lcond, rcond, conds, freshCs)
import TRS.Identifiers (HasName (..), HasMu (..), IdFTRS(..), IdVTRS(..), HasFSort(..), HasSort(..), Sort(..), HasArity(..))

import Data.Set as S (Set, findMax, elems, unions, toList, fromList, map, union, empty, null, member, singleton)
import Data.Maybe (isJust, fromJust, isNothing)
import Control.Monad.State (MonadState (..), runState)

import Text.PrettyPrint.HughesPJClass as Doc (Pretty (..), text, (<+>), (<>), punctuate, hcat, comma, hsep, parens, lparen, rparen, char)

-----------------------------------------------------------------------------
-- Data
-----------------------------------------------------------------------------


data InfCheckerCriteria = 
  InfCheckerCriteria { trsVars          :: [IdVTRS]
                     , cpsConst         :: (Term IdFTRS IdVTRS, Term IdFTRS IdVTRS)
                     , infCheckerJVN    :: String
                     , freshCP          :: Maybe (Term IdFTRS IdVTRS, Term IdFTRS IdVTRS)
                     , freshCPVars      :: [IdVTRS]
                     , condsConst       :: [Cond (Term IdFTRS IdVTRS)]
                     , freshCCPConds    :: [Cond (Term IdFTRS IdVTRS)]
                     , freshCCPCondsVs  :: [IdVTRS]
                     , rCondTs          :: [Term IdFTRS IdVTRS]
                     , rCondJVarNames   :: [String]
                     , crulesICC        :: [CRule (Term IdFTRS IdVTRS)]
                     , rulesICC         :: [Rule (Term IdFTRS IdVTRS)]
                     }


data Prover9Criteria idF idV = 
  Prover9Criteria { congruence      :: [CongruenceTheory idF]
                  , cpNotSVars      :: Maybe (CriticalPair IdFTRS IdVTRS)
                  , ccpVars         :: Maybe [IdVTRS]
                  , cpConsNotSVars  :: (Term IdFTRS IdVTRS, Term IdFTRS IdVTRS)
                  , crulesNotSVars  :: Maybe [CRule (Term IdFTRS IdVTRS)]
                  , rulesNotSVars   :: Maybe [Rule (Term IdFTRS IdVTRS)]
                  , csrCondNotSVars :: Maybe (Term IdFTRS IdVTRS, Term IdFTRS IdVTRS)
                  , ccpJoinVarNames :: [String]
                  , cruleCondsJvs   :: [String]
                  }

data CongruenceTheory idF = 
  CongruenceTheory { func     :: idF
                   , ctStvars :: [String]
                   , ctmu     :: [Int]
                   }

data CritPairInfCheckerInfo idF idV =
  CritPairInfCheckerInfo { cpairI          :: CriticalPairInfo idF idV
                         , cpIcccrit       :: InfCheckerCriteria
                         , cpIOut          :: String
                         , cpInfConds      :: Bool
                         }


-----------------------------------------------------------------------------
-- Functions
-----------------------------------------------------------------------------


buildInfCheckerCriteria :: (HasSignature trs idF idV, HasRules trs (Term idF idV), HasArity idF, HasName idF, HasName idV, Ord idV, Eq idF) => trs -> CriticalPair idF idV -> InfCheckerCriteria
buildInfCheckerCriteria trs cp = InfCheckerCriteria tvs cpC jvn Nothing [] [] [] [] [] [] [] rls
  where variables = getTRSVariables trs
        tvs = Prelude.map sVarsToVars $ elems variables
        cpC = cpVarsToCons $ cpSVarsToVars cp
        jvn = getNonRepeatedVar variables Nothing
        rls = Prelude.map ruleSVarsToVars . elems $ getTRSRules trs


buildInfCheckerCriteriaCTRS :: (HasSignature trs idF idV, HasCRules trs (Term idF idV), HasArity idF, HasName idF, HasName idV, Ord idF, Ord idV, Eq idF, Pretty idV) => trs -> CriticalPair idF idV -> InfCheckerCriteria
buildInfCheckerCriteriaCTRS trs cp
    | Prelude.null cs = InfCheckerCriteria tvs cpC jvn (Just fcp) cpvs [] [] [] [] [] crls []
    | S.null $ condsVars cs = InfCheckerCriteria tvs cpC jvn (Just fcp) cpvs condsC (Prelude.map condSVarsToVars cs) [] [] [] crls []
    | otherwise = InfCheckerCriteria tvs cpC jvn (Just fcp) cpvs condsC (Prelude.map condSVarsToVars freshCds) (toList cvars) [] [] crls []
  where variables = getTRSVariables trs
        tvs = Prelude.map sVarsToVars $ elems variables
        cpC = cpVarsToCons $ cpSVarsToVars cp
        cs = ccpConds cp
        maxid = if S.null variables then 0 else ((+1) . getId . findMax $ variables)
        (freshSTerm,maxid1) = runState (fresh $ s cp) maxid
        (freshTTerm,maxid2) = runState (fresh $ t cp) maxid1
        fcp = (termSVarsToVars freshSTerm, termSVarsToVars freshTTerm)
        cpvs = Prelude.map sVarsToVars $ elems ((vars freshSTerm) `union` (vars freshTTerm))
        condsC = condsVarsToCons cs
        (freshCds,maxid3) = runState (freshCs cs) maxid2
        cvars = S.map sVarsToVars $ condsVars freshCds
        jvn = getNonRepeatedVar variables Nothing
        crls = Prelude.map cruleSVarsToVars . elems $ getCTRSRules trs


buildInfCheckerCriteriaRCondsCRule :: (HasSignature trs idF idV, HasCRules trs (Term idF idV), HasArity idF, HasName idF, HasName idV, Ord idF, Ord idV, Eq idF, Pretty idV) => trs -> InfCheckerCriteria
buildInfCheckerCriteriaRCondsCRule trs
    | Prelude.null cs = InfCheckerCriteria tvs cpC "" Nothing [] [] [] [] [] [] crls []
    | S.null cvars = InfCheckerCriteria tvs cpC "" Nothing [] [] (Prelude.map condSVarsToVars cs) [] ts jvns crls []
    | otherwise = InfCheckerCriteria tvs cpC "" Nothing [] [] (Prelude.map condSVarsToVars cs) (toList cvars) ts jvns crls []
  where variables = getTRSVariables trs
        tvs = Prelude.map sVarsToVars $ elems variables
        crs = elems $ getCTRSRules trs
        cpC = (getMockTerm, getMockTerm)
        cs = concatMap conds crs
        cvars = S.map sVarsToVars . rcondsVars $ Just cs
        --ts = Prelude.map termVarsToCons $ Prelude.map rcond cs
        freshTsFromZero a = freshTs a 0
        ts = Prelude.map termVarsToCons . freshTsFromZero . elems . rcondNotRepeatedTerms $ Just cs
        jvns = getNonRepeatedVars (getTRSVariables trs) Nothing (length ts)
        crls = Prelude.map cruleSVarsToVars . elems $ getCTRSRules trs


getMockTerm :: Term IdFTRS IdVTRS
getMockTerm = V (IdVTRS { vid = 1, vname = Just "x"})


buildProver9Criteria :: (HasSignature trs idF idV, HasCRules trs (Term idF idV), HasArity idF, HasName idF, HasName idV, Ord idV, Eq idF)
                      => trs -> CriticalPair idF idV -> Prover9Criteria idF idV
buildProver9Criteria trs cp = Prover9Criteria ct (Just cpN) (Just cpVs) cpC (Just crC) Nothing Nothing jvs condJoinVs
  where ct = createCongTheories (elems $ getTRSSymbols trs) (getTRSVariables trs)
        cpN = cpSVarsToVars cp
        cpVs = elems (((vars $ s cpN) `union` (vars $ t cpN)) `union` (condsVars $ ccpConds cpN))
        cpC = cpSVarsToCons cp
        crC = Prelude.map cruleSVarsToVars (elems $ getCTRSRules trs)
        jvs = getNonRepeatedVars (getTRSVariables trs) Nothing ((length $ ccpConds cp) + 1)
        ncondJoinVs = maxNumOfConds (elems $ getCTRSRules trs) 0
        condJoinVs = getNonRepeatedVars (getTRSVariables trs) Nothing ncondJoinVs


buildProver9CriteriaCSR :: (HasSignature trs idF idV, HasRules trs (Term idF idV), HasMu idF, HasArity idF, HasName idF, HasName idV, Ord idV, Eq idF) 
                        => trs -> CriticalPair idF idV -> Prover9Criteria idF idV
buildProver9CriteriaCSR trs cp = Prover9Criteria ct Nothing Nothing cpC Nothing (Just rls) (Just (x,y)) jvs []
  where ct = createCongTheoriesCSR (elems $ getTRSSymbols trs) (getTRSVariables trs)
        cpC = cpSVarsToConsVFiltered cp [sx,sy]
        rls = Prelude.map ruleSVarsToVars (elems $ getTRSRules trs)
        jvs = getNonRepeatedVars (getTRSVariables trs) Nothing ((length $ ccpConds cp) + 1)
        (sx :\-> sy) = head $ ccpConds cp
        (x :\-> y) = condSVarsToVars (sx :\-> sy)


createCongTheories :: (HasArity idF, HasName idF, HasName idV) => [idF] -> Set idV -> [CongruenceTheory idF]
createCongTheories [] _ = []
createCongTheories (f:fs) vs = (CongruenceTheory (setName (Just newName) f) (generateCTVars f vs) [1..getFArity f]):(createCongTheories fs vs)
    where name = getName f
          newName = Prelude.map replaceIfApostrophe name

createCongTheoriesCSR :: (HasArity idF, HasMu idF, HasName idF, HasName idV) => [idF] -> Set idV -> [CongruenceTheory idF]
createCongTheoriesCSR [] _ = []
createCongTheoriesCSR (f:fs) vs = (CongruenceTheory (setName (Just newName) f) (generateCTVars f vs) (getFMu f)):(createCongTheoriesCSR fs vs)
    where name = getName f
          newName = Prelude.map replaceIfApostrophe name


getNonRepeatedVar :: (HasName idV) => Set idV -> Maybe Int -> String
getNonRepeatedVar vs Nothing
    | member newN names = getNonRepeatedVar vs (Just 1)
    | otherwise = newN
    where names = S.map getName vs
          newN = "z"
getNonRepeatedVar vs (Just 0)
    | member newN names = getNonRepeatedVar vs (Just 1)
    | otherwise = newN
    where names = S.map getName vs
          newN = "z"
getNonRepeatedVar vs (Just n)
    | member newN names = getNonRepeatedVar vs (Just (n + 1))
    | otherwise = newN
    where names = S.map getName vs
          newN = "z" ++ (show n)


getNonRepeatedVars :: (HasName idV) => Set idV -> Maybe Int -> Int -> [String]
getNonRepeatedVars _ _ 0 = []
getNonRepeatedVars vs Nothing c
    | member newN names = getNonRepeatedVars vs (Just 1) c
    | otherwise = (newN):(getNonRepeatedVars vs (Just 1) (c - 1))
    where names = S.map getName vs
          newN = "z"
getNonRepeatedVars vs (Just n) c 
    | member newN names = getNonRepeatedVars vs (Just (n + 1)) c
    | otherwise = (newN):(getNonRepeatedVars vs (Just (n + 1)) (c - 1))
    where names = S.map getName vs
          newN = "z" ++ (show n)


getNonRepeatedVarNum :: (HasName idV) => Set idV -> Maybe Int -> (String, Maybe Int)
getNonRepeatedVarNum vs Nothing
    | member newN names = getNonRepeatedVarNum vs (Just 1)
    | otherwise = (newN, Just 1)
    where names = S.map getName vs
          newN = "z"
getNonRepeatedVarNum vs (Just n)
    | member newN names = getNonRepeatedVarNum vs (Just (n + 1))
    | otherwise = (newN, Just (n + 1))
    where names = S.map getName vs
          newN = "z" ++ (show n)


getJoinTerminationCRules :: (HasName idV, Ord idV) => [CRule (Term idF idV)] -> [idV] -> Maybe Int -> [(Rule (Term idF idV), [(Cond (Term idF idV), String)])]
getJoinTerminationCRules [] _ _ = []
getJoinTerminationCRules (cr:crs) vs n
  | Prelude.null cs = (r, []):(getJoinTerminationCRules crs vs n)
  | otherwise = (r, getJoinTerminationConds cs vs n):(getJoinTerminationCRules crs vs newN)
  where (r :=> cs) = cr
        newN
          | isNothing n = Just (length cs)
          | otherwise = Just ((fromJust n) + (length cs))


getJoinTerminationConds :: (HasName idV, Ord idV) => [Cond (Term idF idV)] -> [idV] -> Maybe Int -> [(Cond (Term idF idV), String)]
getJoinTerminationConds [] _ _ = []
getJoinTerminationConds (cd:cds) vs n = ((cd),s):(getJoinTerminationConds cds vs nn)
  where (s, nn) = getNonRepeatedVarNum (S.fromList vs) n


generateCTVars :: (HasArity idF, HasName idV) => idF -> Set idV -> [String]
generateCTVars f vs = getNonRepeatedVars vs Nothing (getFArity f)


condsVars :: (Ord idF, Ord idV) => [Cond (Term idF idV)] -> Set idV
condsVars cs = unions . S.elems . S.map condVars $ fromList cs

condVars :: (Ord idF, Ord idV) => Cond (Term idF idV) -> Set idV
condVars c = union (vars $ lcond c) (vars $ rcond c)

rcondsVars :: (Ord idF, Ord idV) => Maybe [Cond (Term idF idV)] -> Set idV
rcondsVars Nothing = S.empty
rcondsVars (Just cs) = unions . S.elems . S.map rcondVars $ fromList cs

rcondVars :: (Ord idF, Ord idV) => Cond (Term idF idV) -> Set idV
rcondVars c = vars $ rcond c

rcondNotRepeatedTerms :: (Ord idF, Ord idV) => Maybe [Cond (Term idF idV)] -> Set (Term idF idV)
rcondNotRepeatedTerms Nothing = S.empty
rcondNotRepeatedTerms (Just cs) = unions . S.elems . S.map rcondTerm $ fromList cs

rcondTerm :: (Ord idF, Ord idV) => Cond (Term idF idV) -> Set (Term idF idV)
rcondTerm c = singleton $ rcond c


freshTs :: (HasName idV, Pretty idV, Ord idF, Ord idV) => [Term idF idV] -> Int -> [Term idF idV]
freshTs [] _ = []
freshTs (x:xs) maxid = freshTerm:(freshTs xs maxid2)
  where (freshTerm,maxid2) = runState (fresh x) maxid


cpVarsToCons :: (HasName idF, HasArity idF, HasName idV) => CriticalPair idF idV -> (Term IdFTRS IdVTRS, Term IdFTRS IdVTRS)
cpVarsToCons c = (termVarsToCons $ s c, termVarsToCons $ t c)

condsVarsToCons :: (HasName idF, HasArity idF, HasName idV) => [Cond (Term idF idV)] -> [Cond (Term IdFTRS IdVTRS)]
condsVarsToCons [] = []
condsVarsToCons (c:cs) = (fmap termVarsToCons c):(condsVarsToCons cs)

termVarsToCons :: (HasName idF, HasArity idF, HasName idV) => Term idF idV -> Term IdFTRS IdVTRS
termVarsToCons (V v) = F IdFTRS{fid = (getId v), fname = ("c_"++(getName v)), arity = 0} []
termVarsToCons (F f ts) = F IdFTRS{fid = (getId f), fname = (getName f), arity = (getFArity f)} (termsVarsIterateToCons ts)

termVarsToConsVFiltered :: (HasName idF, HasArity idF, HasName idV) => Term idF idV -> [Term idF idV] -> Term IdFTRS IdVTRS
termVarsToConsVFiltered t@(V v) vs 
  | or $ Prelude.map (\(V v1) -> (getName v) == (getName v1)) vs = V IdVTRS{vid = (getId v), vname = Just (getName v)}
  | otherwise = F IdFTRS{fid = (getId v), fname = ("c_"++(getName v)), arity = 0} []
termVarsToConsVFiltered (F f ts) vs  = F IdFTRS{fid = (getId f), fname = (getName f), arity = (getFArity f)} (termsVarsIterateToConsVFilt ts vs)

cpSVarsToCons :: (HasName idF, HasArity idF, HasName idV) => CriticalPair idF idV -> (Term IdFTRS IdVTRS, Term IdFTRS IdVTRS)
cpSVarsToCons c = (termVarsToCons . termSVarsToVars $ s c, termVarsToCons . termSVarsToVars $ t c)

cpSVarsToConsVFiltered :: (HasName idF, HasArity idF, HasName idV) => CriticalPair idF idV -> [Term idF idV] -> (Term IdFTRS IdVTRS, Term IdFTRS IdVTRS)
cpSVarsToConsVFiltered c vs = (vsToConsfilt . termSVarsToVars $ s c, vsToConsfilt . termSVarsToVars $ t c)
  where vsTr = Prelude.map (\(V v) -> V IdVTRS{vid = (getId v), vname = Just (getName v)}) vs
        vsToConsfilt t = termVarsToConsVFiltered t vsTr

cpSVarsToVars :: (HasName idF, HasArity idF, HasName idV) => CriticalPair idF idV -> CriticalPair IdFTRS IdVTRS
cpSVarsToVars c  = CriticalPair (termSVarsToVars $ s c) (termSVarsToVars $ t c) (cpPos c) (Prelude.map condSVarsToVars $ ccpConds c) (normWeight c)

cruleSVarsToVars :: (HasName idF, HasArity idF, HasName idV) => CRule (Term idF idV) -> CRule (Term IdFTRS IdVTRS)
cruleSVarsToVars (r :=> cs) = (ruleSVarsToVars r) :=> (Prelude.map condSVarsToVars cs)

ruleSVarsToVars :: (HasName idF, HasArity idF, HasName idV) => Rule (Term idF idV) -> Rule (Term IdFTRS IdVTRS)
ruleSVarsToVars (l :-> r) = (termSVarsToVars l) :-> (termSVarsToVars r)

condSVarsToVars :: (HasName idF, HasArity idF, HasName idV) => Cond (Term idF idV) -> Cond (Term IdFTRS IdVTRS)
condSVarsToVars c = fmap termSVarsToVars c

termSVarsToVars :: (HasName idF, HasArity idF, HasName idV) => Term idF idV -> Term IdFTRS IdVTRS
termSVarsToVars (V v) = V (sVarsToVars v)
termSVarsToVars (F f ts) = F IdFTRS{fid = (getId f), fname = newName, arity = (getFArity f)} (termsVarsIterateToVars ts)
    where name = getName f
          newName = Prelude.map replaceIfApostrophe name

sVarsToVars :: (HasName idV) => idV -> IdVTRS
sVarsToVars v = IdVTRS{vid = (getId v), vname = Just newName}
  where name = getName v
        newName = Prelude.map replaceIfApostrophe name


replaceIfApostrophe :: Char -> Char
replaceIfApostrophe c
    | c == '\'' = '1'
    | otherwise = c


termsVarsIterateToCons :: (HasName idF, HasArity idF, HasName idV) => [Term idF idV] -> [Term IdFTRS IdVTRS]
termsVarsIterateToCons [] = []
termsVarsIterateToCons (t:ts) = (termVarsToCons t):(termsVarsIterateToCons ts)


termsVarsIterateToConsVFilt :: (HasName idF, HasArity idF, HasName idV) => [Term idF idV] -> [Term idF idV] -> [Term IdFTRS IdVTRS]
termsVarsIterateToConsVFilt [] _ = []
termsVarsIterateToConsVFilt (t:ts) vs  = (termVarsToConsVFiltered t vs):(termsVarsIterateToConsVFilt ts vs)


termsVarsIterateToVars :: (HasName idF, HasArity idF, HasName idV) => [Term idF idV] -> [Term IdFTRS IdVTRS]
termsVarsIterateToVars [] = []
termsVarsIterateToVars (t:ts) = (termSVarsToVars t):(termsVarsIterateToVars ts)



-- Pretty
{-
instance (Pretty idF) => Pretty (CongruenceTheory idF) where
  pPrint (CongruenceTheory 
    { func = f
    , ctStvars = vs
    }) = (hsep . punctuate (char '\n') . Prelude.map (\v1 -> 
            text "red(x,y) -> red" Doc.<> lparen Doc.<> pPrint f
            Doc.<> (parens . hcat . punctuate comma . Prelude.map(\v2 -> 
                if v1 == v2 then text "x" else text v2) 
            $ vs) Doc.<> comma
            Doc.<> pPrint f
            Doc.<> (parens . hcat . punctuate comma . Prelude.map(\v2 -> 
                if v1 == v2 then text "y" else text v2) 
            $ vs) Doc.<> rparen Doc.<> char '.'
        ) $ vs)
-}

instance (Pretty idF) => Pretty (CongruenceTheory idF) where
  pPrint (CongruenceTheory 
    { func = f
    , ctStvars = vs
    , ctmu = mu
    }) = (hsep . punctuate (char '\n') . Prelude.map (\m -> 
            text "red(x,y) -> red" Doc.<> lparen Doc.<> pPrint f
            Doc.<> (parens . hcat . punctuate comma . Prelude.map text
              $ (replaceNElem (m-1) "x" vs)) Doc.<> comma
            Doc.<> pPrint f
            Doc.<> (parens . hcat . punctuate comma . Prelude.map text
              $ (replaceNElem (m-1) "y" vs))
            Doc.<> rparen Doc.<> char '.'
        ) $ mu)

ff = IdFTRS { fid = 1, fname = "f", arity = 2}
fp = IdFTRS { fid = 1, fname = "f'", arity = 2}
cc = IdFTRS { fid = 2, fname = "c", arity = 0}
gg = IdFTRS { fid = 3, fname = "g", arity = 3}
v1 = IdVTRS { vid = 1, vname = Just "x"}
v2 = IdVTRS { vid = 2, vname = Just "y"}
