{-# LANGUAGE TypeFamilies #-}
{-# LANGUAGE FlexibleInstances #-}
{-# LANGUAGE MultiParamTypeClasses #-}
-----------------------------------------------------------------------------
-- |
-- Module      :  Framework.Problem.CPCRewriting
-- Copyright   :  (c) muterm development team
-- License     :  see LICENSE
--
-- Maintainer  :  rgutierrez@dsic.upv.es
-- Stability   :  unstable
-- Portability :  non-portable
--
-- This module contains the instances of a CPCRewriting problem
--
-----------------------------------------------------------------------------

module Framework.Problem.CPCRewriting (

-- * Exported data

CPCRewriting (..)

) where

import MuTerm.Framework.Problem (IsProblem (..), MkProblem (..))
import Framework.Problems (PScheme, HasPScheme(..))

import Text.PrettyPrint.HughesPJClass (Pretty (..))

-----------------------------------------------------------------------------
-- Data
-----------------------------------------------------------------------------

-- | CPCRewriting problem
data CPCRewriting  = CPCRewriting PScheme  deriving (Eq, Ord, Show)

-----------------------------------------------------------------------------
-- Instances
-----------------------------------------------------------------------------

-- | CPCRewriting problem is a Problem
instance IsProblem CPCRewriting where
  data Problem CPCRewriting a = CPCRew PScheme a deriving (Eq, Ord, Show)
  getProblemType (CPCRew m _) = CPCRewriting m
  getR (CPCRew _ r) = r

-- | We make a rewriting problem
instance MkProblem CPCRewriting trs where 
    mkProblem (CPCRewriting m) = CPCRew m
    mapR f (CPCRew m r) = CPCRew m (f r)

-- | Problem CPCRewriting must be a functor
instance Functor (Problem CPCRewriting) where fmap f (CPCRew m r) = CPCRew m (f r)

instance (Pretty a) => Pretty (Problem CPCRewriting a) where
    pPrint = pPrint . getR

instance HasPScheme CPCRewriting where
    getPScheme (CPCRewriting m) = m
    setPScheme m (CPCRew _ r) = CPCRew m r
