{-# LANGUAGE TypeFamilies #-}
{-# LANGUAGE FlexibleInstances #-}
{-# LANGUAGE MultiParamTypeClasses #-}
-----------------------------------------------------------------------------
-- |
-- Module      :  Framework.Problem.CPLCRewriting
-- Copyright   :  (c) muterm development team
-- License     :  see LICENSE
--
-- Maintainer  :  rgutierrez@dsic.upv.es
-- Stability   :  unstable
-- Portability :  non-portable
--
-- This module contains the instances of a CPLCRewriting problem
--
-----------------------------------------------------------------------------

module Framework.Problem.CPLCRewriting (

-- * Exported data

CPLCRewriting (..)

) where

import MuTerm.Framework.Problem (IsProblem (..), MkProblem (..))
import Framework.Problems (PScheme, HasPScheme(..))

import Text.PrettyPrint.HughesPJClass (Pretty (..))

-----------------------------------------------------------------------------
-- Data
-----------------------------------------------------------------------------

-- | CPLCRewriting problem
data CPLCRewriting  = CPLCRewriting PScheme  deriving (Eq, Ord, Show)

-----------------------------------------------------------------------------
-- Instances
-----------------------------------------------------------------------------

-- | CPLCRewriting problem is a Problem
instance IsProblem CPLCRewriting where
  data Problem CPLCRewriting a = CPLCRew PScheme a deriving (Eq, Ord, Show)
  getProblemType (CPLCRew m _) = CPLCRewriting m
  getR (CPLCRew _ r) = r

-- | We make a rewriting problem
instance MkProblem CPLCRewriting trs where 
    mkProblem (CPLCRewriting m) = CPLCRew m
    mapR f (CPLCRew m r) = CPLCRew m (f r)

-- | Problem CPLCRewriting must be a functor
instance Functor (Problem CPLCRewriting) where fmap f (CPLCRew m r) = CPLCRew m (f r)

instance (Pretty a) => Pretty (Problem CPLCRewriting a) where
    pPrint = pPrint . getR

instance HasPScheme CPLCRewriting where
    getPScheme (CPLCRewriting m) = m
    setPScheme m (CPLCRew _ r) = CPLCRew m r
