{-# LANGUAGE TypeFamilies #-}
{-# LANGUAGE FlexibleInstances #-}
{-# LANGUAGE MultiParamTypeClasses #-}
-----------------------------------------------------------------------------
-- |
-- Module      :  Framework.Problem.CPLRewriting
-- Copyright   :  (c) muterm development team
-- License     :  see LICENSE
--
-- Maintainer  :  rgutierrez@dsic.upv.es
-- Stability   :  unstable
-- Portability :  non-portable
--
-- This module contains the instances of a CPLRewriting problem
--
-----------------------------------------------------------------------------

module Framework.Problem.CPLRewriting (

-- * Exported data

CPLRewriting (..)

) where

import MuTerm.Framework.Problem (IsProblem (..), MkProblem (..))
import Framework.Problems (PScheme, HasPScheme(..))

import Text.PrettyPrint.HughesPJClass (Pretty (..))

-----------------------------------------------------------------------------
-- Data
-----------------------------------------------------------------------------

-- | CPLRewriting problem
data CPLRewriting  = CPLRewriting PScheme  deriving (Eq, Ord, Show)

-----------------------------------------------------------------------------
-- Instances
-----------------------------------------------------------------------------

-- | CPLRewriting problem is a Problem
instance IsProblem CPLRewriting where
  data Problem CPLRewriting a = CPLRew PScheme a deriving (Eq, Ord, Show)
  getProblemType (CPLRew m _) = CPLRewriting m
  getR (CPLRew _ r) = r

-- | We make a rewriting problem
instance MkProblem CPLRewriting trs where 
    mkProblem (CPLRewriting m) = CPLRew m
    mapR f (CPLRew m r) = CPLRew m (f r)

-- | Problem CPLRewriting must be a functor
instance Functor (Problem CPLRewriting) where fmap f (CPLRew m r) = CPLRew m (f r)

instance (Pretty a) => Pretty (Problem CPLRewriting a) where
    pPrint = pPrint . getR

instance HasPScheme CPLRewriting where
    getPScheme (CPLRewriting m) = m
    setPScheme m (CPLRew _ r) = CPLRew m r
