{-# LANGUAGE TypeFamilies #-}
{-# LANGUAGE FlexibleInstances #-}
{-# LANGUAGE MultiParamTypeClasses #-}
-----------------------------------------------------------------------------
-- |
-- Module      :  Framework.Problem.CPNCRewriting
-- Copyright   :  (c) muterm development team
-- License     :  see LICENSE
--
-- Maintainer  :  rgutierrez@dsic.upv.es
-- Stability   :  unstable
-- Portability :  non-portable
--
-- This module contains the instances of a CPNCRewriting problem
--
-----------------------------------------------------------------------------

module Framework.Problem.CPNCRewriting (

-- * Exported data

CPNCRewriting (..)

) where

import MuTerm.Framework.Problem (IsProblem (..), MkProblem (..))
import Framework.Problems (PScheme, HasPScheme(..))

import Text.PrettyPrint.HughesPJClass (Pretty (..))

-----------------------------------------------------------------------------
-- Data
-----------------------------------------------------------------------------

-- | CPNCRewriting problem
data CPNCRewriting  = CPNCRewriting PScheme  deriving (Eq, Ord, Show)

-----------------------------------------------------------------------------
-- Instances
-----------------------------------------------------------------------------

-- | CPNCRewriting problem is a Problem
instance IsProblem CPNCRewriting where
  data Problem CPNCRewriting a = CPNCRew PScheme a deriving (Eq, Ord, Show)
  getProblemType (CPNCRew m _) = CPNCRewriting m
  getR (CPNCRew _ r) = r

-- | We make a rewriting problem
instance MkProblem CPNCRewriting trs where 
    mkProblem (CPNCRewriting m) = CPNCRew m
    mapR f (CPNCRew m r) = CPNCRew m (f r)

-- | Problem CPNCRewriting must be a functor
instance Functor (Problem CPNCRewriting) where fmap f (CPNCRew m r) = CPNCRew m (f r)

instance (Pretty a) => Pretty (Problem CPNCRewriting a) where
    pPrint = pPrint . getR

instance HasPScheme CPNCRewriting where
    getPScheme (CPNCRewriting m) = m
    setPScheme m (CPNCRew _ r) = CPNCRew m r
