{-# LANGUAGE TypeFamilies #-}
{-# LANGUAGE FlexibleInstances #-}
{-# LANGUAGE MultiParamTypeClasses #-}
-----------------------------------------------------------------------------
-- |
-- Module      :  Framework.Problem.CPNRewriting
-- Copyright   :  (c) muterm development team
-- License     :  see LICENSE
--
-- Maintainer  :  rgutierrez@dsic.upv.es
-- Stability   :  unstable
-- Portability :  non-portable
--
-- This module contains the instances of a CPNRewriting problem
--
-----------------------------------------------------------------------------

module Framework.Problem.CPNRewriting (

-- * Exported data

CPNRewriting (..)

) where

import MuTerm.Framework.Problem (IsProblem (..), MkProblem (..))
import Framework.Problems (PScheme, HasPScheme(..))

import Text.PrettyPrint.HughesPJClass (Pretty (..))

-----------------------------------------------------------------------------
-- Data
-----------------------------------------------------------------------------

-- | CPNRewriting problem
data CPNRewriting  = CPNRewriting PScheme  deriving (Eq, Ord, Show)

-----------------------------------------------------------------------------
-- Instances
-----------------------------------------------------------------------------

-- | CPNRewriting problem is a Problem
instance IsProblem CPNRewriting where
  data Problem CPNRewriting a = CPNRew PScheme a deriving (Eq, Ord, Show)
  getProblemType (CPNRew m _) = CPNRewriting m
  getR (CPNRew _ r) = r

-- | We make a rewriting problem
instance MkProblem CPNRewriting trs where 
    mkProblem (CPNRewriting m) = CPNRew m
    mapR f (CPNRew m r) = CPNRew m (f r)

-- | Problem CPNRewriting must be a functor
instance Functor (Problem CPNRewriting) where fmap f (CPNRew m r) = CPNRew m (f r)

instance (Pretty a) => Pretty (Problem CPNRewriting a) where
    pPrint = pPrint . getR

instance HasPScheme CPNRewriting where
    getPScheme (CPNRewriting m) = m
    setPScheme m (CPNRew _ r) = CPNRew m r
