{-# LANGUAGE TypeFamilies #-}
{-# LANGUAGE FlexibleInstances #-}
{-# LANGUAGE MultiParamTypeClasses #-}
-----------------------------------------------------------------------------
-- |
-- Module      :  Framework.Problem.CPPCCRewriting
-- Copyright   :  (c) muterm development team
-- License     :  see LICENSE
--
-- Maintainer  :  rgutierrez@dsic.upv.es
-- Stability   :  unstable
-- Portability :  non-portable
--
-- This module contains the instances of a CPPCCRewriting problem
--
-----------------------------------------------------------------------------

module Framework.Problem.CPPCCRewriting (

-- * Exported data

CPPCCRewriting (..)

) where

import MuTerm.Framework.Problem (IsProblem (..), MkProblem (..))
import Framework.Problems (PScheme, HasPScheme(..))

import Text.PrettyPrint.HughesPJClass (Pretty (..))

-----------------------------------------------------------------------------
-- Data
-----------------------------------------------------------------------------

-- | CPPCCRewriting problem
data CPPCCRewriting  = CPPCCRewriting PScheme  deriving (Eq, Ord, Show)

-----------------------------------------------------------------------------
-- Instances
-----------------------------------------------------------------------------

-- | CPPCCRewriting problem is a Problem
instance IsProblem CPPCCRewriting where
  data Problem CPPCCRewriting a = CPPCCRew PScheme a deriving (Eq, Ord, Show)
  getProblemType (CPPCCRew m _) = CPPCCRewriting m
  getR (CPPCCRew _ r) = r

-- | We make a rewriting problem
instance MkProblem CPPCCRewriting trs where 
    mkProblem (CPPCCRewriting m) = CPPCCRew m
    mapR f (CPPCCRew m r) = CPPCCRew m (f r)

-- | Problem CPPCCRewriting must be a functor
instance Functor (Problem CPPCCRewriting) where fmap f (CPPCCRew m r) = CPPCCRew m (f r)

instance (Pretty a) => Pretty (Problem CPPCCRewriting a) where
    pPrint = pPrint . getR

instance HasPScheme CPPCCRewriting where
    getPScheme (CPPCCRewriting m) = m
    setPScheme m (CPPCCRew _ r) = CPPCCRew m r
