{-# LANGUAGE TypeFamilies #-}
{-# LANGUAGE FlexibleInstances #-}
{-# LANGUAGE MultiParamTypeClasses #-}
-----------------------------------------------------------------------------
-- |
-- Module      :  Framework.Problem.CPPCRewriting
-- Copyright   :  (c) muterm development team
-- License     :  see LICENSE
--
-- Maintainer  :  rgutierrez@dsic.upv.es
-- Stability   :  unstable
-- Portability :  non-portable
--
-- This module contains the instances of a CPPCRewriting problem
--
-----------------------------------------------------------------------------

module Framework.Problem.CPPCRewriting (

-- * Exported data

CPPCRewriting (..)

) where

import MuTerm.Framework.Problem (IsProblem (..), MkProblem (..))
import Framework.Problems (PScheme, HasPScheme(..))

import Text.PrettyPrint.HughesPJClass (Pretty (..))

-----------------------------------------------------------------------------
-- Data
-----------------------------------------------------------------------------

-- | CPPCRewriting problem
data CPPCRewriting  = CPPCRewriting PScheme  deriving (Eq, Ord, Show)

-----------------------------------------------------------------------------
-- Instances
-----------------------------------------------------------------------------

-- | CPPCRewriting problem is a Problem
instance IsProblem CPPCRewriting where
  data Problem CPPCRewriting a = CPPCRew PScheme a deriving (Eq, Ord, Show)
  getProblemType (CPPCRew m _) = CPPCRewriting m
  getR (CPPCRew _ r) = r

-- | We make a rewriting problem
instance MkProblem CPPCRewriting trs where 
    mkProblem (CPPCRewriting m) = CPPCRew m
    mapR f (CPPCRew m r) = CPPCRew m (f r)

-- | Problem CPPCRewriting must be a functor
instance Functor (Problem CPPCRewriting) where fmap f (CPPCRew m r) = CPPCRew m (f r)

instance (Pretty a) => Pretty (Problem CPPCRewriting a) where
    pPrint = pPrint . getR

instance HasPScheme CPPCRewriting where
    getPScheme (CPPCRewriting m) = m
    setPScheme m (CPPCRew _ r) = CPPCRew m r
