{-# LANGUAGE TypeFamilies #-}
{-# LANGUAGE FlexibleInstances #-}
{-# LANGUAGE MultiParamTypeClasses #-}
-----------------------------------------------------------------------------
-- |
-- Module      :  Framework.Problem.CPRewriting
-- Copyright   :  (c) muterm development team
-- License     :  see LICENSE
--
-- Maintainer  :  rgutierrez@dsic.upv.es
-- Stability   :  unstable
-- Portability :  non-portable
--
-- This module contains the instances of a CPRewriting problem
--
-----------------------------------------------------------------------------

module Framework.Problem.CPRewriting (

-- * Exported data

CPRewriting (..)

) where

import MuTerm.Framework.Problem (IsProblem (..), MkProblem (..))
import Framework.Problems (PScheme, HasPScheme(..))

import Text.PrettyPrint.HughesPJClass (Pretty (..))

-----------------------------------------------------------------------------
-- Data
-----------------------------------------------------------------------------

-- | CPRewriting problem
data CPRewriting  = CPRewriting PScheme  deriving (Eq, Ord, Show)

-----------------------------------------------------------------------------
-- Instances
-----------------------------------------------------------------------------

-- | CPRewriting problem is a Problem
instance IsProblem CPRewriting where
  data Problem CPRewriting a = CPRew PScheme a deriving (Eq, Ord, Show)
  getProblemType (CPRew m _) = CPRewriting m
  getR (CPRew _ r) = r

-- | We make a rewriting problem
instance MkProblem CPRewriting trs where 
    mkProblem (CPRewriting m) = CPRew m
    mapR f (CPRew m r) = CPRew m (f r)

-- | Problem CPRewriting must be a functor
instance Functor (Problem CPRewriting) where fmap f (CPRew m r) = CPRew m (f r)

instance (Pretty a) => Pretty (Problem CPRewriting a) where
    pPrint = pPrint . getR

instance HasPScheme CPRewriting where
    getPScheme (CPRewriting m) = m
    setPScheme m (CPRew _ r) = CPRew m r
