{-# LANGUAGE TypeFamilies #-}
{-# LANGUAGE FlexibleInstances #-}
{-# LANGUAGE MultiParamTypeClasses #-}
-----------------------------------------------------------------------------
-- |
-- Module      :  Framework.Problem.CPTCRewriting
-- Copyright   :  (c) muterm development team
-- License     :  see LICENSE
--
-- Maintainer  :  rgutierrez@dsic.upv.es
-- Stability   :  unstable
-- Portability :  non-portable
--
-- This module contains the instances of a CPTCRewriting problem
--
-----------------------------------------------------------------------------

module Framework.Problem.CPTCRewriting (

-- * Exported data

CPTCRewriting (..)

) where

import MuTerm.Framework.Problem (IsProblem (..), MkProblem (..))
import Framework.Problems (PScheme, HasPScheme(..))

import Text.PrettyPrint.HughesPJClass (Pretty (..))

-----------------------------------------------------------------------------
-- Data
-----------------------------------------------------------------------------

-- | CPTCRewriting problem
data CPTCRewriting  = CPTCRewriting PScheme  deriving (Eq, Ord, Show)

-----------------------------------------------------------------------------
-- Instances
-----------------------------------------------------------------------------

-- | CPTCRewriting problem is a Problem
instance IsProblem CPTCRewriting where
  data Problem CPTCRewriting a = CPTCRew PScheme a deriving (Eq, Ord, Show)
  getProblemType (CPTCRew m _) = CPTCRewriting m
  getR (CPTCRew _ r) = r

-- | We make a rewriting problem
instance MkProblem CPTCRewriting trs where 
    mkProblem (CPTCRewriting m) = CPTCRew m
    mapR f (CPTCRew m r) = CPTCRew m (f r)

-- | Problem CPTCRewriting must be a functor
instance Functor (Problem CPTCRewriting) where fmap f (CPTCRew m r) = CPTCRew m (f r)

instance (Pretty a) => Pretty (Problem CPTCRewriting a) where
    pPrint = pPrint . getR

instance HasPScheme CPTCRewriting where
    getPScheme (CPTCRewriting m) = m
    setPScheme m (CPTCRew _ r) = CPTCRew m r
