{-# LANGUAGE TypeFamilies #-}
{-# LANGUAGE FlexibleInstances #-}
{-# LANGUAGE MultiParamTypeClasses #-}
-----------------------------------------------------------------------------
-- |
-- Module      :  Framework.Problem.CPTPCCRewriting
-- Copyright   :  (c) muterm development team
-- License     :  see LICENSE
--
-- Maintainer  :  rgutierrez@dsic.upv.es
-- Stability   :  unstable
-- Portability :  non-portable
--
-- This module contains the instances of a CPTPCCRewriting problem
--
-----------------------------------------------------------------------------

module Framework.Problem.CPTPCCRewriting (

-- * Exported data

CPTPCCRewriting (..)

) where

import MuTerm.Framework.Problem (IsProblem (..), MkProblem (..))
import Framework.Problems (PScheme, HasPScheme(..))

import Text.PrettyPrint.HughesPJClass (Pretty (..))

-----------------------------------------------------------------------------
-- Data
-----------------------------------------------------------------------------

-- | CPTPCCRewriting problem
data CPTPCCRewriting  = CPTPCCRewriting PScheme  deriving (Eq, Ord, Show)

-----------------------------------------------------------------------------
-- Instances
-----------------------------------------------------------------------------

-- | CPTPCCRewriting problem is a Problem
instance IsProblem CPTPCCRewriting where
  data Problem CPTPCCRewriting a = CPTPCCRew PScheme a deriving (Eq, Ord, Show)
  getProblemType (CPTPCCRew m _) = CPTPCCRewriting m
  getR (CPTPCCRew _ r) = r

-- | We make a rewriting problem
instance MkProblem CPTPCCRewriting trs where 
    mkProblem (CPTPCCRewriting m) = CPTPCCRew m
    mapR f (CPTPCCRew m r) = CPTPCCRew m (f r)

-- | Problem CPTPCCRewriting must be a functor
instance Functor (Problem CPTPCCRewriting) where fmap f (CPTPCCRew m r) = CPTPCCRew m (f r)

instance (Pretty a) => Pretty (Problem CPTPCCRewriting a) where
    pPrint = pPrint . getR

instance HasPScheme CPTPCCRewriting where
    getPScheme (CPTPCCRewriting m) = m
    setPScheme m (CPTPCCRew _ r) = CPTPCCRew m r
