{-# LANGUAGE TypeFamilies #-}
{-# LANGUAGE FlexibleInstances #-}
{-# LANGUAGE MultiParamTypeClasses #-}
-----------------------------------------------------------------------------
-- |
-- Module      :  Framework.Problem.CPTPCRewriting
-- Copyright   :  (c) muterm development team
-- License     :  see LICENSE
--
-- Maintainer  :  rgutierrez@dsic.upv.es
-- Stability   :  unstable
-- Portability :  non-portable
--
-- This module contains the instances of a CPTPCRewriting problem
--
-----------------------------------------------------------------------------

module Framework.Problem.CPTPCRewriting (

-- * Exported data

CPTPCRewriting (..)

) where

import MuTerm.Framework.Problem (IsProblem (..), MkProblem (..))
import Framework.Problems (PScheme, HasPScheme(..))

import Text.PrettyPrint.HughesPJClass (Pretty (..))

-----------------------------------------------------------------------------
-- Data
-----------------------------------------------------------------------------

-- | CPTPCRewriting problem
data CPTPCRewriting  = CPTPCRewriting PScheme  deriving (Eq, Ord, Show)

-----------------------------------------------------------------------------
-- Instances
-----------------------------------------------------------------------------

-- | CPTPCRewriting problem is a Problem
instance IsProblem CPTPCRewriting where
  data Problem CPTPCRewriting a = CPTPCRew PScheme a deriving (Eq, Ord, Show)
  getProblemType (CPTPCRew m _) = CPTPCRewriting m
  getR (CPTPCRew _ r) = r

-- | We make a rewriting problem
instance MkProblem CPTPCRewriting trs where 
    mkProblem (CPTPCRewriting m) = CPTPCRew m
    mapR f (CPTPCRew m r) = CPTPCRew m (f r)

-- | Problem CPTPCRewriting must be a functor
instance Functor (Problem CPTPCRewriting) where fmap f (CPTPCRew m r) = CPTPCRew m (f r)

instance (Pretty a) => Pretty (Problem CPTPCRewriting a) where
    pPrint = pPrint . getR

instance HasPScheme CPTPCRewriting where
    getPScheme (CPTPCRewriting m) = m
    setPScheme m (CPTPCRew _ r) = CPTPCRew m r
