{-# LANGUAGE TypeFamilies #-}
{-# LANGUAGE FlexibleInstances #-}
{-# LANGUAGE MultiParamTypeClasses #-}
-----------------------------------------------------------------------------
-- |
-- Module      :  Framework.Problem.CPTRewriting
-- Copyright   :  (c) muterm development team
-- License     :  see LICENSE
--
-- Maintainer  :  rgutierrez@dsic.upv.es
-- Stability   :  unstable
-- Portability :  non-portable
--
-- This module contains the instances of a CPTRewriting problem
--
-----------------------------------------------------------------------------

module Framework.Problem.CPTRewriting (

-- * Exported data

CPTRewriting (..)

) where

import MuTerm.Framework.Problem (IsProblem (..), MkProblem (..))
import Framework.Problems (PScheme, HasPScheme(..))

import Text.PrettyPrint.HughesPJClass (Pretty (..))

-----------------------------------------------------------------------------
-- Data
-----------------------------------------------------------------------------

-- | CPTRewriting problem
data CPTRewriting  = CPTRewriting PScheme  deriving (Eq, Ord, Show)

-----------------------------------------------------------------------------
-- Instances
-----------------------------------------------------------------------------

-- | CPTRewriting problem is a Problem
instance IsProblem CPTRewriting where
  data Problem CPTRewriting a = CPTRew PScheme a deriving (Eq, Ord, Show)
  getProblemType (CPTRew m _) = CPTRewriting m
  getR (CPTRew _ r) = r

-- | We make a rewriting problem
instance MkProblem CPTRewriting trs where 
    mkProblem (CPTRewriting m) = CPTRew m
    mapR f (CPTRew m r) = CPTRew m (f r)

-- | Problem CPTRewriting must be a functor
instance Functor (Problem CPTRewriting) where fmap f (CPTRew m r) = CPTRew m (f r)

instance (Pretty a) => Pretty (Problem CPTRewriting a) where
    pPrint = pPrint . getR

instance HasPScheme CPTRewriting where
    getPScheme (CPTRewriting m) = m
    setPScheme m (CPTRew _ r) = CPTRew m r
