{-# LANGUAGE TypeFamilies #-}
{-# LANGUAGE FlexibleInstances #-}
{-# LANGUAGE MultiParamTypeClasses #-}
-----------------------------------------------------------------------------
-- |
-- Module      :  Framework.Problem.CRewriting
-- Copyright   :  (c) muterm development team
-- License     :  see LICENSE
--
-- Maintainer  :  rgutierrez@dsic.upv.es
-- Stability   :  unstable
-- Portability :  non-portable
--
-- This module contains the instances of a Conditional Rewriting problem
--
-----------------------------------------------------------------------------

module Framework.Problem.CRewriting (

-- * Exported data

CRewriting (..)

) where

import MuTerm.Framework.Problem (IsProblem (..), MkProblem (..))
import Framework.Problems (PScheme, HasPScheme(..))

import Text.PrettyPrint.HughesPJClass (Pretty (..))

-----------------------------------------------------------------------------
-- Data
-----------------------------------------------------------------------------

-- | Conditional Rewriting problem
data CRewriting  = CRewriting PScheme  deriving (Eq, Ord, Show)

-----------------------------------------------------------------------------
-- Instances
-----------------------------------------------------------------------------

-- | ConditoCnal Rewriting problem is a Problem
instance IsProblem CRewriting where
  data Problem CRewriting a = CRew PScheme a deriving (Eq, Ord, Show)
  getProblemType (CRew m _) = CRewriting m
  getR (CRew _ r) = r

-- | We make a rewriting problem
instance MkProblem CRewriting trs where
    mkProblem (CRewriting m) = CRew m
    mapR f (CRew m r) = CRew m (f r)

-- | Problem CRewriting must be a functor
instance Functor (Problem CRewriting) where fmap f (CRew m r) = CRew m (f r)

instance (Pretty a) => Pretty (Problem CRewriting a) where
    pPrint = pPrint . getR

instance HasPScheme CRewriting where
    getPScheme    (CRewriting m) = m
    setPScheme m (CRew _ r) = CRew m r
