{-# LANGUAGE TypeFamilies #-}
{-# LANGUAGE FlexibleInstances #-}
{-# LANGUAGE MultiParamTypeClasses #-}
-----------------------------------------------------------------------------
-- |
-- Module      :  Framework.Problem.LCRewriting
-- Copyright   :  (c) muterm development team
-- License     :  see LICENSE
--
-- Maintainer  :  rgutierrez@dsic.upv.es
-- Stability   :  unstable
-- Portability :  non-portable
--
-- This module contains the instances of a LCRewriting problem
--
-----------------------------------------------------------------------------

module Framework.Problem.LCRewriting (

-- * Exported data

LCRewriting (..)

) where

import MuTerm.Framework.Problem (IsProblem (..), MkProblem (..))
import Framework.Problems (PScheme, HasPScheme(..))

import Text.PrettyPrint.HughesPJClass (Pretty (..))

-----------------------------------------------------------------------------
-- Data
-----------------------------------------------------------------------------

-- | LCRewriting problem
data LCRewriting  = LCRewriting PScheme  deriving (Eq, Ord, Show)

-----------------------------------------------------------------------------
-- Instances
-----------------------------------------------------------------------------

-- | LCRewriting problem is a Problem
instance IsProblem LCRewriting where
  data Problem LCRewriting a = LCRew PScheme a deriving (Eq, Ord, Show)
  getProblemType (LCRew m _) = LCRewriting m
  getR (LCRew _ r) = r

-- | We make a rewriting problem
instance MkProblem LCRewriting trs where 
    mkProblem (LCRewriting m) = LCRew m
    mapR f (LCRew m r) = LCRew m (f r)

-- | Problem LCRewriting must be a functor
instance Functor (Problem LCRewriting) where fmap f (LCRew m r) = LCRew m (f r)

instance (Pretty a) => Pretty (Problem LCRewriting a) where
    pPrint = pPrint . getR

instance HasPScheme LCRewriting where
    getPScheme (LCRewriting m) = m
    setPScheme m (LCRew _ r) = LCRew m r
