{-# LANGUAGE TypeFamilies #-}
{-# LANGUAGE FlexibleInstances #-}
{-# LANGUAGE MultiParamTypeClasses #-}
-----------------------------------------------------------------------------
-- |
-- Module      :  Framework.Problem.LRewriting
-- Copyright   :  (c) muterm development team
-- License     :  see LICENSE
--
-- Maintainer  :  rgutierrez@dsic.upv.es
-- Stability   :  unstable
-- Portability :  non-portable
--
-- This module contains the instances of a LRewriting problem
--
-----------------------------------------------------------------------------

module Framework.Problem.LRewriting (

-- * Exported data

LRewriting (..)

) where

import MuTerm.Framework.Problem (IsProblem (..), MkProblem (..))
import Framework.Problems (PScheme, HasPScheme(..))

import Text.PrettyPrint.HughesPJClass (Pretty (..))

-----------------------------------------------------------------------------
-- Data
-----------------------------------------------------------------------------

-- | LRewriting problem
data LRewriting  = LRewriting PScheme  deriving (Eq, Ord, Show)

-----------------------------------------------------------------------------
-- Instances
-----------------------------------------------------------------------------

-- | LRewriting problem is a Problem
instance IsProblem LRewriting where
  data Problem LRewriting a = LRew PScheme a deriving (Eq, Ord, Show)
  getProblemType (LRew m _) = LRewriting m
  getR (LRew _ r) = r

-- | We make a rewriting problem
instance MkProblem LRewriting trs where 
    mkProblem (LRewriting m) = LRew m
    mapR f (LRew m r) = LRew m (f r)

-- | Problem LRewriting must be a functor
instance Functor (Problem LRewriting) where fmap f (LRew m r) = LRew m (f r)

instance (Pretty a) => Pretty (Problem LRewriting a) where
    pPrint = pPrint . getR

instance HasPScheme LRewriting where
    getPScheme (LRewriting m) = m
    setPScheme m (LRew _ r) = LRew m r
