{-# LANGUAGE TypeFamilies #-}
{-# LANGUAGE FlexibleInstances #-}
{-# LANGUAGE MultiParamTypeClasses #-}
-----------------------------------------------------------------------------
-- |
-- Module      :  Framework.Problem.NCRewriting
-- Copyright   :  (c) muterm development team
-- License     :  see LICENSE
--
-- Maintainer  :  rgutierrez@dsic.upv.es
-- Stability   :  unstable
-- Portability :  non-portable
--
-- This module contains the instances of a NCRewriting problem
--
-----------------------------------------------------------------------------

module Framework.Problem.NCRewriting (

-- * Exported data

NCRewriting (..)

) where

import MuTerm.Framework.Problem (IsProblem (..), MkProblem (..))
import Framework.Problems (PScheme, HasPScheme(..))

import Text.PrettyPrint.HughesPJClass (Pretty (..))

-----------------------------------------------------------------------------
-- Data
-----------------------------------------------------------------------------

-- | NCRewriting problem
data NCRewriting  = NCRewriting PScheme  deriving (Eq, Ord, Show)

-----------------------------------------------------------------------------
-- Instances
-----------------------------------------------------------------------------

-- | NCRewriting problem is a Problem
instance IsProblem NCRewriting where
  data Problem NCRewriting a = NCRew PScheme a deriving (Eq, Ord, Show)
  getProblemType (NCRew m _) = NCRewriting m
  getR (NCRew _ r) = r

-- | We make a rewriting problem
instance MkProblem NCRewriting trs where 
    mkProblem (NCRewriting m) = NCRew m
    mapR f (NCRew m r) = NCRew m (f r)

-- | Problem NCRewriting must be a functor
instance Functor (Problem NCRewriting) where fmap f (NCRew m r) = NCRew m (f r)

instance (Pretty a) => Pretty (Problem NCRewriting a) where
    pPrint = pPrint . getR

instance HasPScheme NCRewriting where
    getPScheme (NCRewriting m) = m
    setPScheme m (NCRew _ r) = NCRew m r
