{-# LANGUAGE TypeFamilies #-}
{-# LANGUAGE FlexibleInstances #-}
{-# LANGUAGE MultiParamTypeClasses #-}
-----------------------------------------------------------------------------
-- |
-- Module      :  Framework.Problem.NRewriting
-- Copyright   :  (c) muterm development team
-- License     :  see LICENSE
--
-- Maintainer  :  rgutierrez@dsic.upv.es
-- Stability   :  unstable
-- Portability :  non-portable
--
-- This module contains the instances of a NRewriting problem
--
-----------------------------------------------------------------------------

module Framework.Problem.NRewriting (

-- * Exported data

NRewriting (..)

) where

import MuTerm.Framework.Problem (IsProblem (..), MkProblem (..))
import Framework.Problems (PScheme, HasPScheme(..))

import Text.PrettyPrint.HughesPJClass (Pretty (..))

-----------------------------------------------------------------------------
-- Data
-----------------------------------------------------------------------------

-- | NRewriting problem
data NRewriting  = NRewriting PScheme  deriving (Eq, Ord, Show)

-----------------------------------------------------------------------------
-- Instances
-----------------------------------------------------------------------------

-- | NRewriting problem is a Problem
instance IsProblem NRewriting where
  data Problem NRewriting a = NRew PScheme a deriving (Eq, Ord, Show)
  getProblemType (NRew m _) = NRewriting m
  getR (NRew _ r) = r

-- | We make a rewriting problem
instance MkProblem NRewriting trs where 
    mkProblem (NRewriting m) = NRew m
    mapR f (NRew m r) = NRew m (f r)

-- | Problem NRewriting must be a functor
instance Functor (Problem NRewriting) where fmap f (NRew m r) = NRew m (f r)

instance (Pretty a) => Pretty (Problem NRewriting a) where
    pPrint = pPrint . getR

instance HasPScheme NRewriting where
    getPScheme (NRewriting m) = m
    setPScheme m (NRew _ r) = NRew m r
