{-# LANGUAGE TypeFamilies #-}
{-# LANGUAGE FlexibleInstances #-}
{-# LANGUAGE MultiParamTypeClasses #-}
-----------------------------------------------------------------------------
-- |
-- Module      :  Framework.Problem.Rewriting
-- Copyright   :  (c) muterm development team
-- License     :  see LICENSE
--
-- Maintainer  :  rgutierrez@dsic.upv.es
-- Stability   :  unstable
-- Portability :  non-portable
--
-- This module contains the instances of a Rewriting problem
--
-----------------------------------------------------------------------------

module Framework.Problem.Rewriting (

-- * Exported data

Rewriting (..)

) where

import MuTerm.Framework.Problem (IsProblem (..), MkProblem (..))
import Framework.Problems (PScheme, HasPScheme(..))

import Text.PrettyPrint.HughesPJClass (Pretty (..))

-----------------------------------------------------------------------------
-- Data
-----------------------------------------------------------------------------

-- | Rewriting problem
data Rewriting  = Rewriting PScheme  deriving (Eq, Ord, Show)

-----------------------------------------------------------------------------
-- Instances
-----------------------------------------------------------------------------

-- | Rewriting problem is a Problem
instance IsProblem Rewriting where
  data Problem Rewriting a = Rew PScheme a deriving (Eq, Ord, Show)
  getProblemType (Rew m _) = Rewriting m
  getR (Rew _ r) = r

-- | We make a rewriting problem
instance MkProblem Rewriting trs where 
    mkProblem (Rewriting m) = Rew m
    mapR f (Rew m r) = Rew m (f r)

-- | Problem Rewriting must be a functor
instance Functor (Problem Rewriting) where fmap f (Rew m r) = Rew m (f r)

instance (Pretty a) => Pretty (Problem Rewriting a) where
    pPrint = pPrint . getR

instance HasPScheme Rewriting where
    getPScheme    (Rewriting m) = m
    setPScheme m (Rew _ r) = Rew m r
