{-# LANGUAGE TypeFamilies #-}
{-# LANGUAGE FlexibleInstances #-}
{-# LANGUAGE MultiParamTypeClasses #-}
-----------------------------------------------------------------------------
-- |
-- Module      :  Framework.Problem.TCRewriting
-- Copyright   :  (c) muterm development team
-- License     :  see LICENSE
--
-- Maintainer  :  rgutierrez@dsic.upv.es
-- Stability   :  unstable
-- Portability :  non-portable
--
-- This module contains the instances of a TCRewriting problem
--
-----------------------------------------------------------------------------

module Framework.Problem.TCRewriting (

-- * Exported data

TCRewriting (..)

) where

import MuTerm.Framework.Problem (IsProblem (..), MkProblem (..))
import Framework.Problems (PScheme, HasPScheme(..))

import Text.PrettyPrint.HughesPJClass (Pretty (..))

-----------------------------------------------------------------------------
-- Data
-----------------------------------------------------------------------------

-- | TCRewriting problem
data TCRewriting  = TCRewriting PScheme  deriving (Eq, Ord, Show)

-----------------------------------------------------------------------------
-- Instances
-----------------------------------------------------------------------------

-- | TCRewriting problem is a Problem
instance IsProblem TCRewriting where
  data Problem TCRewriting a = TCRew PScheme a deriving (Eq, Ord, Show)
  getProblemType (TCRew m _) = TCRewriting m
  getR (TCRew _ r) = r

-- | We make a rewriting problem
instance MkProblem TCRewriting trs where 
    mkProblem (TCRewriting m) = TCRew m
    mapR f (TCRew m r) = TCRew m (f r)

-- | Problem TCRewriting must be a functor
instance Functor (Problem TCRewriting) where fmap f (TCRew m r) = TCRew m (f r)

instance (Pretty a) => Pretty (Problem TCRewriting a) where
    pPrint = pPrint . getR

instance HasPScheme TCRewriting where
    getPScheme (TCRewriting m) = m
    setPScheme m (TCRew _ r) = TCRew m r
