{-# LANGUAGE TypeFamilies #-}
{-# LANGUAGE FlexibleInstances #-}
{-# LANGUAGE MultiParamTypeClasses #-}
-----------------------------------------------------------------------------
-- |
-- Module      :  Framework.Problem.TRewriting
-- Copyright   :  (c) muterm development team
-- License     :  see LICENSE
--
-- Maintainer  :  rgutierrez@dsic.upv.es
-- Stability   :  unstable
-- Portability :  non-portable
--
-- This module contains the instances of a TRewriting problem
--
-----------------------------------------------------------------------------

module Framework.Problem.TRewriting (

-- * Exported data

TRewriting (..)

) where

import MuTerm.Framework.Problem (IsProblem (..), MkProblem (..))
import Framework.Problems (PScheme, HasPScheme(..))

import Text.PrettyPrint.HughesPJClass (Pretty (..))

-----------------------------------------------------------------------------
-- Data
-----------------------------------------------------------------------------

-- | TRewriting problem
data TRewriting  = TRewriting PScheme  deriving (Eq, Ord, Show)

-----------------------------------------------------------------------------
-- Instances
-----------------------------------------------------------------------------

-- | TRewriting problem is a Problem
instance IsProblem TRewriting where
  data Problem TRewriting a = TRew PScheme a deriving (Eq, Ord, Show)
  getProblemType (TRew m _) = TRewriting m
  getR (TRew _ r) = r

-- | We make a rewriting problem
instance MkProblem TRewriting trs where 
    mkProblem (TRewriting m) = TRew m
    mapR f (TRew m r) = TRew m (f r)

-- | Problem TRewriting must be a functor
instance Functor (Problem TRewriting) where fmap f (TRew m r) = TRew m (f r)

instance (Pretty a) => Pretty (Problem TRewriting a) where
    pPrint = pPrint . getR

instance HasPScheme TRewriting where
    getPScheme (TRewriting m) = m
    setPScheme m (TRew _ r) = TRew m r
