{-# LANGUAGE MultiParamTypeClasses #-}
{-# LANGUAGE FlexibleInstances #-}
{-# LANGUAGE FlexibleContexts #-}
{-# LANGUAGE TypeFamilies #-}
{-# LANGUAGE UndecidableInstances #-}
{-# LANGUAGE NoMonomorphismRestriction #-}
-----------------------------------------------------------------------------
-- |
-- Module      :  Framework.Processor.CRewriting.CCPsDistrCProcessor
-- Copyright   :  (c) muterm development team
-- License     :  see LICENSE
-- 
-- Maintainer  :  rgutierrez@dsic.upv.es
-- Stability   :  unstable
-- Portability :  non-portable 
--
-- This module applies the processor that distribute a list of CCPs
--
-----------------------------------------------------------------------------

module Framework.Processor.CRewriting.CCPsDistrCProcessor (

-- * Exported data


-- * Exported functions

ccPairsDistrProc

) where

import TRS.TRSs(HasSignature (..), HasCRules (..), HasRules (..), HasOSSignature(..))
import TRS.Terms(Term(..), subTerm, mgu, applySigma, replace, isLinear, vars)
import TRS.Identifiers (HasName (..), IdFTRS(..), IdVTRS(..), HasFSort(..), HasSort(..), Sort(..))
import Framework.Problem.Types(InConfCSCTRS (..), InCPCSCTRS (..), InTermCSCTRS (..), InCPTermPCCSCTRS (..))
import Framework.Problems(IsConfProblem (..), IsCPProblem (..), IsCPTermPCProblem (..), CriticalPair (..), CriticalPairInfo (..), 
                          getPSchemeFromProblem)
import MuTerm.Framework.Problem (Problem (..), MkProblem, mkProblem)
import MuTerm.Framework.Processor (Processor (..))
import MuTerm.Framework.Proof (success, dontKnow, andP, Proof, Info (..), isSuccess, refuted, runProof, singleP)
import Framework.Problem.CPCRewriting (CPCRewriting (..))
import Framework.Problem.CPTPCCRewriting (CPTPCCRewriting (..))
import Framework.ProblemConclusions (ProblemConclusionsCTRS (..), problemConclusionCTRS)

import Text.PrettyPrint.HughesPJClass as Doc (Pretty (..), text, (<+>), (<>), punctuate, hcat, comma, braces, brackets, parens, hsep)
import Control.Monad.State (MonadState (..), runState)
import Control.Monad (MonadPlus)
import Data.Map as M (Map, toList)
import Data.Maybe (isJust, fromJust, isNothing)

-----------------------------------------------------------------------------
-- Data
-----------------------------------------------------------------------------

-- | Critical Pairs Processor
data CCPsDistrCProcessor = CCPsDistrCProcessor

-- | The information obtained are the rules
data CCPsDistrCProcInfo problem = 
  CCPsDistrCProcInfo { inProblem :: problem }
  | CCPsDistrDontKnow { inProblem :: problem } 


-----------------------------------------------------------------------------
-- Instances
-----------------------------------------------------------------------------

-- Processor

instance {-# OVERLAPPABLE #-} (
    conftrs ~ InCPCSCTRS trs (CriticalPairInfo idF idV)
  , IsConfProblem conftrs trs
  , IsCPProblem conftrs (CriticalPairInfo idF idV)
  , HasSignature trs idF idV
  , HasCRules trs (Term idF idV)
  , Pretty idF, Pretty idV, Pretty trs
  , HasName idV
  , Eq idF, Eq idV
  , Ord idF, Ord idV
  , Info info (CCPsDistrCProcInfo (Problem CPCRewriting conftrs))
  ) => Processor info CCPsDistrCProcessor (Problem CPCRewriting conftrs) (Problem CPCRewriting conftrs) where
  apply CCPsDistrCProcessor inP
    | null cpis = dontKnow (CCPsDistrDontKnow inP) inP
    | otherwise = andP (CCPsDistrCProcInfo inP) inP 
                   [mkProblem (CPCRewriting (getPSchemeFromProblem inP)) (InCPCSCTRS trs [cp]) | cp <- cpis]
    where trs = getConfProblem . getR $ inP
          cpis = getCPairs . getR $ inP

instance {-# OVERLAPPABLE #-} (
    conftrs ~ InCPTermPCCSCTRS trs (CriticalPairInfo idF idV) (Maybe(Bool)) ProblemConclusionsCTRS
  , IsConfProblem conftrs trs
  , IsCPTermPCProblem conftrs (CriticalPairInfo idF idV) (Maybe(Bool)) ProblemConclusionsCTRS
  , HasSignature trs idF idV
  , HasCRules trs (Term idF idV)
  , Pretty idF, Pretty idV, Pretty trs
  , HasName idV
  , Eq idF, Eq idV
  , Ord idF, Ord idV, Ord String
  , Info info (CCPsDistrCProcInfo (Problem CPTPCCRewriting conftrs))
  ) => Processor info CCPsDistrCProcessor (Problem CPTPCCRewriting conftrs) (Problem CPTPCCRewriting conftrs) where
  apply CCPsDistrCProcessor inP
    | null cpis = dontKnow (CCPsDistrDontKnow inP) inP
    | otherwise = andP (CCPsDistrCProcInfo inP) inP 
                   [mkProblem (CPTPCCRewriting (getPSchemeFromProblem inP)) (InCPTermPCCSCTRS trs [cp] term pconc) | cp <- cpis]
    where trs = getConfProblem . getR $ inP
          cpis = getCPairsCPTPC . getR $ inP
          term = getTermCPTPC . getR $ inP
          pconc = getProbConcCPTPC . getR $ inP


-- Pretty

instance {-# OVERLAPPABLE #-} (
    IsConfProblem conftrs trs
  , HasSignature trs idF idV
  , HasCRules trs (Term idF idV)
  , HasName idV
  , Pretty idF, Pretty idV, Pretty trs
  , Pretty (Problem CPCRewriting conftrs)
  ) => Pretty (CCPsDistrCProcInfo (Problem CPCRewriting conftrs)) where
  pPrint CCPsDistrCProcInfo { inProblem = problem}
    = pPrint problem
      Doc.<> text "\nConditional Critical Pairs Distributor Processor"

instance {-# OVERLAPPABLE #-} (
    IsConfProblem conftrs trs
  , HasSignature trs idF idV
  , HasCRules trs (Term idF idV)
  , HasName idV
  , Pretty idF, Pretty idV, Pretty trs
  , Pretty (Problem CPTPCCRewriting conftrs)
  ) => Pretty (CCPsDistrCProcInfo (Problem CPTPCCRewriting conftrs)) where
  pPrint CCPsDistrCProcInfo { inProblem = problem}
    = pPrint problem
      Doc.<> text "\nConditional Critical Pairs Distributor Processor"



-----------------------------------------------------------------------------
-- Functions
-----------------------------------------------------------------------------

-- | Applies the Conditional Critical Pairs Distributor Processor
ccPairsDistrProc :: ( Processor info CCPsDistrCProcessor o d
            , MonadPlus mp
            , Info info o
            , Info info d) => o -> Proof info mp d
ccPairsDistrProc  = apply CCPsDistrCProcessor
