{-# LANGUAGE MultiParamTypeClasses #-}
{-# LANGUAGE FlexibleInstances #-}
{-# LANGUAGE FlexibleContexts #-}
{-# LANGUAGE TypeFamilies #-}
{-# LANGUAGE UndecidableInstances #-}
{-# LANGUAGE NoMonomorphismRestriction #-}
-----------------------------------------------------------------------------
-- |
-- Module      :  Framework.Processor.IdProcessor
-- Copyright   :  (c) muterm development team
-- License     :  see LICENSE
-- 
-- Maintainer  :  rgutierrez@dsic.upv.es
-- Stability   :  unstable
-- Portability :  non-portable 
--
-- This module apply the id processor
--
-----------------------------------------------------------------------------

module Framework.Processor.IdProcessor (

-- * Exported functions

idProc

) where

import TRS.TRSs(HasSignature (..))
import TRS.Terms(Term)
import Framework.Problems (IsConfProblem)
import MuTerm.Framework.Problem (Problem (..), MkProblem)
import MuTerm.Framework.Processor (Processor (..))
import MuTerm.Framework.Proof (singleP, Proof, Info (..))

import Text.PrettyPrint.HughesPJClass as Doc (Pretty (..), text, (<+>), (<>))
import Control.Monad (MonadPlus)

-----------------------------------------------------------------------------
-- Data
-----------------------------------------------------------------------------

-- | The extra variables processor check if there is a rule with extra
-- variables
data IdProcessor  = IdProcessor

-- | The information obtained are the rules
data IdProcInfo problem = IdProcInfo { inProblem :: problem
                                     }

-----------------------------------------------------------------------------
-- Instances
-----------------------------------------------------------------------------

-- Processor

instance (
  IsConfProblem conftrs trs
  , HasSignature trs idF idV
  , Pretty idF, Pretty idV, Pretty trs
  , Pretty (Problem tproblem conftrs)
  , Info info (IdProcInfo (Problem tproblem conftrs))
  ) => Processor info IdProcessor (Problem tproblem conftrs) (Problem tproblem conftrs) where
  apply IdProcessor inP
    = singleP (IdProcInfo inP) inP inP

-- Pretty

instance (
  IsConfProblem conftrs trs
  , HasSignature trs idF idV
  , Pretty idF, Pretty idV, Pretty trs
  , Pretty (Problem tproblem conftrs)
  ) => Pretty (IdProcInfo (Problem tproblem conftrs)) where
  pPrint IdProcInfo { inProblem    = problem }
    = text "\n\n" Doc.<> pPrint problem

-----------------------------------------------------------------------------
-- Functions
-----------------------------------------------------------------------------

-- | Applies the Identity Processor
idProc :: ( Processor info IdProcessor o d
          , MonadPlus mp
          , Info info o
          , Info info d) => o -> Proof info mp d
idProc  = apply IdProcessor
