{-# LANGUAGE MultiParamTypeClasses #-}
{-# LANGUAGE FlexibleInstances #-}
{-# LANGUAGE FlexibleContexts #-}
{-# LANGUAGE TypeFamilies #-}
{-# LANGUAGE UndecidableInstances #-}
{-# LANGUAGE NoMonomorphismRestriction #-}
{-# LANGUAGE CPP #-}
-----------------------------------------------------------------------------
-- |
-- Module      :  Framework.Processor.Rewriting.CSTRSProcessor
-- Copyright   :  (c) muterm development team
-- License     :  see LICENSE
-- 
-- Maintainer  :  rgutierrez@dsic.upv.es
-- Stability   :  unstable
-- Portability :  non-portable 
--
-- This module apply the processor that checks if R is a CS-TRS
--
-----------------------------------------------------------------------------

module Framework.Processor.Rewriting.CSTRSProcessor (

-- * Exported functions

cstrsProc

) where

import TRS.TRSs(HasSignature (..), HasRules (..), HasOSSignature(..))
import TRS.Terms(Term)
import TRS.Identifiers (HasArity (..), HasMu (..))
import TRS.Rules(Rule(..), lhs, rhs)
import TRS.Terms(Term(..))
import Framework.Problem.Types(InConfCSTRS (..),)
import Framework.Problem.Rewriting (Rewriting (..))
import Framework.Problems (IsConfProblem (..), getPSchemeFromProblem)
import Framework.TRSAux (isLHRV, isCSTRSSymb)
import MuTerm.Framework.Problem (Problem (..), MkProblem, mkProblem)
import MuTerm.Framework.Processor (Processor (..))
import MuTerm.Framework.Proof (dontKnow, success, singleP, Proof, Info (..))

import Text.PrettyPrint.HughesPJClass as Doc (Pretty (..), text, (<+>), (<>))
import Control.Monad (MonadPlus)
import Data.Set as S (Set, elems)

-----------------------------------------------------------------------------
-- Data
-----------------------------------------------------------------------------


data CSTRSProcessor  = CSTRSProcessor

data CSTRSProcInfo problem = 
    CSTRSProcInfo { inProblem :: problem }
  | CSTRSProcDontKnow { inProblem :: problem }

-----------------------------------------------------------------------------
-- Instances
-----------------------------------------------------------------------------

-- Processor

instance (
  conftrs ~ InConfCSTRS trs
  , IsConfProblem conftrs trs
  , HasSignature trs idF idV
  , HasRules trs (Term idF idV)
  , HasArity idF
  , HasMu idF
  , Eq idF, Eq idV
  , Ord idF, Ord idV
  , Pretty idF, Pretty idV, Pretty trs
  , Pretty (Problem Rewriting conftrs)
  , Info info (CSTRSProcInfo (Problem Rewriting conftrs))
  ) => Processor info CSTRSProcessor (Problem Rewriting conftrs) (Problem Rewriting conftrs) where
  apply CSTRSProcessor inP
    | cstrs = singleP (CSTRSProcInfo inP) inP inP
    | otherwise = dontKnow (CSTRSProcDontKnow inP) inP
    where trs = getConfProblem . getR $ inP
          cstrs = or . map isCSTRSSymb . elems $ getTRSSymbols trs

-- Pretty

instance (
  IsConfProblem conftrs trs
  , HasSignature trs idF idV
  , HasRules trs (Term idF idV)
  , Pretty idF, Pretty idV, Pretty trs
  , Pretty (Problem Rewriting conftrs)
  ) => Pretty (CSTRSProcInfo (Problem Rewriting conftrs)) where
  pPrint CSTRSProcInfo { inProblem    = problem}
    = text "\nCS-TRS Processor:\n"
      Doc.<> text "R is a CS-TRS"

-----------------------------------------------------------------------------
-- Functions
-----------------------------------------------------------------------------

-- | Applies the CS-TRS Processor
cstrsProc :: (Processor info CSTRSProcessor o d
          , MonadPlus mp
          , Info info o
          , Info info d) => o -> Proof info mp d
cstrsProc  = apply CSTRSProcessor
