{-# LANGUAGE MultiParamTypeClasses #-}
{-# LANGUAGE FlexibleInstances #-}
{-# LANGUAGE FlexibleContexts #-}
{-# LANGUAGE TypeFamilies #-}
{-# LANGUAGE UndecidableInstances #-}
{-# LANGUAGE NoMonomorphismRestriction #-}
{-# LANGUAGE CPP #-}
-----------------------------------------------------------------------------
-- |
-- Module      :  Framework.Processor.Rewriting.CleanTRSProcessor
-- Copyright   :  (c) muterm development team
-- License     :  see LICENSE
-- 
-- Maintainer  :  rgutierrez@dsic.upv.es
-- Stability   :  unstable
-- Portability :  non-portable 
--
-- This module apply the cleanTRSProc processor
--
-----------------------------------------------------------------------------

module Framework.Processor.Rewriting.CleanTRSProcessor (

-- * Exported functions

cleanTRSProc

) where

import TRS.TRSs(HasSignature (..), HasRules (..), HasOSSignature(..))
import TRS.Terms(Term)
import TRS.Identifiers (HasName (..), IdFTRS(..), IdVTRS(..), HasFSort(..), HasSort(..), Sort(..))
import TRS.Rules(Rule(..), lhs, rhs)
import TRS.Terms(Term(..))
import Framework.Problem.Types(InConfCSTRS (..),)
import Framework.Problem.Rewriting (Rewriting (..))
import Framework.Problems (IsConfProblem (..), getPSchemeFromProblem)
import MuTerm.Framework.Problem (Problem (..), MkProblem, mkProblem)
import MuTerm.Framework.Processor (Processor (..))
import MuTerm.Framework.Proof (singleP, Proof, Info (..))

import Text.PrettyPrint.HughesPJClass as Doc (Pretty (..), text, (<+>), (<>))
import Control.Monad (MonadPlus)
import Data.Set as S (Set, elems, fromList)

-----------------------------------------------------------------------------
-- Data
-----------------------------------------------------------------------------

-- | The extra variables processor check if there is a rule with extra
-- variables
data CleanTRSProcessor  = CleanTRSProcessor

-- | The information obtained are the rules
data CleanTRSProcInfo problem = CleanTRSProcInfo { inProblem :: problem }

-----------------------------------------------------------------------------
-- Instances
-----------------------------------------------------------------------------

-- Processor

instance (
    conftrs ~ InConfCSTRS trs
  , IsConfProblem conftrs trs
  , HasSignature trs idF idV
  , HasRules trs (Term idF idV)
  , Eq idF, Eq idV
  , Ord idF, Ord idV
  , Pretty idF, Pretty idV, Pretty trs
  , Pretty (Problem Rewriting conftrs)
  , Info info (CleanTRSProcInfo (Problem Rewriting conftrs))
  ) => Processor info CleanTRSProcessor (Problem Rewriting conftrs) (Problem Rewriting conftrs) where
  apply CleanTRSProcessor inP
    = singleP (CleanTRSProcInfo newP) inP newP 
    where trs = getConfProblem . getR $ inP
          rules = elems . getTRSRules $ trs
          newRules = fromList $ cleanRules rules
          newTRS = setTRSRules newRules trs
          newP = mkProblem (Rewriting (getPSchemeFromProblem inP)) (InConfCSTRS newTRS)

-- Pretty

instance (
  IsConfProblem conftrs trs
  , HasSignature trs idF idV
  , HasRules trs (Term idF idV)
  , Pretty idF, Pretty idV, Pretty trs
  , Pretty (Problem Rewriting conftrs)
  ) => Pretty (CleanTRSProcInfo (Problem Rewriting conftrs)) where
  pPrint CleanTRSProcInfo { inProblem    = problem }
    = text "\n\nCleanTRS Processor:\n"
      Doc.<> pPrint problem

-----------------------------------------------------------------------------
-- Functions
-----------------------------------------------------------------------------

-- | Applies the Clean TRS Processor
cleanTRSProc :: (Processor info CleanTRSProcessor o d
          , MonadPlus mp
          , Info info o
          , Info info d) => o -> Proof info mp d
cleanTRSProc  = apply CleanTRSProcessor


cleanRules :: (Eq idF, Eq idV) => [Rule (Term idF idV)] -> [Rule (Term idF idV)]
cleanRules [] = []
cleanRules (r:rs) = nrl ++ (cleanRules rs)
  where nrl = if lhs r == rhs r then [] else [r]