{-# LANGUAGE MultiParamTypeClasses #-}
{-# LANGUAGE FlexibleInstances #-}
{-# LANGUAGE FlexibleContexts #-}
{-# LANGUAGE TypeFamilies #-}
{-# LANGUAGE UndecidableInstances #-}
{-# LANGUAGE NoMonomorphismRestriction #-}
{-# LANGUAGE CPP #-}
-----------------------------------------------------------------------------
-- |
-- Module      :  Framework.Processor.Rewriting.LinearityProcessor
-- Copyright   :  (c) muterm development team
-- License     :  see LICENSE
-- 
-- Maintainer  :  rgutierrez@dsic.upv.es
-- Stability   :  unstable
-- Portability :  non-portable 
--
-- This module apply the processor that tries to check convergence using ground 
-- terms.
--
-----------------------------------------------------------------------------

module Framework.Processor.Rewriting.LinearityProcessor (

-- * Exported functions

linProc

) where

import TRS.TRSs(HasSignature (..), HasRules (..), HasOSSignature(..))
import TRS.Terms(Term(..))
import TRS.Identifiers (HasName (..), IdFTRS(..), IdVTRS(..), HasFSort(..), HasSort(..), Sort(..))
import Framework.Problem.Types(InConfCSTRS (..), InTermCSTRS (..), InLinCSTRS (..))
import Framework.Problems(IsConfProblem (..), IsTProblem (..), getPSchemeFromProblem, IsLProblem (..))
import Framework.TRSAux (isTRSLinear)
import MuTerm.Framework.Problem (Problem (..), MkProblem, mkProblem)
import MuTerm.Framework.Processor (Processor (..))
import MuTerm.Framework.Proof (success, dontKnow, andP, Proof, Info (..), isSuccess, refuted, runProof, singleP)
import TRS.Rules(Rule(..))
import Framework.Problem.Rewriting (Rewriting (..))
import Framework.Problem.LRewriting (LRewriting (..))

import Text.PrettyPrint.HughesPJClass as Doc (Pretty (..), text, (<+>), (<>), punctuate, hcat)
import Control.Monad (MonadPlus)

-----------------------------------------------------------------------------
-- Data
-----------------------------------------------------------------------------

-- | Linearity Processor
data LinearityProcessor = LinearityProcessor

-- | The information obtained are the rules
data LinProcInfo problem = 
  LinProcInfo { inProblem :: problem
              , inLin     :: Bool
              }
  | LinDontKnow { inProblem :: problem
                , inLin     :: Bool
                } 


-----------------------------------------------------------------------------
-- Instances
-----------------------------------------------------------------------------

-- Processor

instance {-# OVERLAPPABLE #-} (
    conftrs1 ~ InConfCSTRS trs
  , conftrs2 ~ InLinCSTRS trs Bool
  , IsConfProblem conftrs1 trs
  , IsConfProblem conftrs2 trs
  , IsLProblem conftrs2 Bool
  , HasSignature trs idF idV
  , HasRules trs (Term idF idV)
  , Pretty idF, Pretty idV, Pretty trs
  , HasName idV
  , Eq idF, Eq idV
  , Ord idV
  , Info info (LinProcInfo (Problem Rewriting conftrs1))
  ) => Processor info LinearityProcessor (Problem Rewriting conftrs1) (Problem LRewriting conftrs2) where
  apply LinearityProcessor inP
    = andP (LinProcInfo inP lin) inP [mkProblem (LRewriting (getPSchemeFromProblem inP)) (InLinCSTRS trs lin)] --singleP (LinProcInfo inP lin) inP (mkProblem (LRewriting (getPSchemeFromProblem inP)) (InLinCSTRS trs lin))--success (LinProcInfo inP lin) inP -- --andP (LinProcInfo inP lin) inP [mkProblem (LRewriting (getPSchemeFromProblem inP)) (InLinCSTRS trs lin)]
    where trs = getConfProblem . getR $ inP
          lin = isTRSLinear trs

          
-- Pretty

instance {-# OVERLAPPABLE #-} (
    IsConfProblem conftrs trs
  , HasSignature trs idF idV
  , HasRules trs (Term idF idV)
  , HasName idV
  , Pretty idF, Pretty idV, Pretty trs
  , Pretty (Problem Rewriting conftrs)
  ) => Pretty (LinProcInfo (Problem Rewriting conftrs)) where
  pPrint LinProcInfo { inProblem = problem
                      , inLin = lin
                      }
    = text "\nLinearity Processor:\n"
      Doc.<> text "Linear?\n"
      Doc.<> text (if lin then "YES" else "NO")
    where trs = getConfProblem . getR $ problem


-----------------------------------------------------------------------------
-- Functions
-----------------------------------------------------------------------------

-- | Applies the Linearity Processor
linProc :: ( Processor info LinearityProcessor o d
            , MonadPlus mp
            , Info info o
            , Info info d) => o -> Proof info mp d
linProc  = apply LinearityProcessor