{-# LANGUAGE MultiParamTypeClasses #-}
{-# LANGUAGE FlexibleInstances #-}
{-# LANGUAGE FlexibleContexts #-}
{-# LANGUAGE TypeFamilies #-}
{-# LANGUAGE UndecidableInstances #-}
{-# LANGUAGE NoMonomorphismRestriction #-}
{-# LANGUAGE CPP #-}
-----------------------------------------------------------------------------
-- |
-- Module      :  Framework.Processor.Rewriting.NotCSTRSProcessor
-- Copyright   :  (c) muterm development team
-- License     :  see LICENSE
-- 
-- Maintainer  :  rgutierrez@dsic.upv.es
-- Stability   :  unstable
-- Portability :  non-portable 
--
-- This module apply the processor that checks if R is not a CS-TRS
--
-----------------------------------------------------------------------------

module Framework.Processor.Rewriting.NotCSTRSProcessor (

-- * Exported functions

notcstrsProc

) where

import TRS.TRSs(HasSignature (..), HasRules (..), HasOSSignature(..))
import TRS.Terms(Term)
import TRS.Identifiers (HasArity (..), HasMu (..))
import TRS.Rules(Rule(..), lhs, rhs)
import TRS.Terms(Term(..))
import Framework.Problem.Types(InConfCSTRS (..),)
import Framework.Problem.Rewriting (Rewriting (..))
import Framework.Problems (IsConfProblem (..), getPSchemeFromProblem)
import Framework.TRSAux (isLHRV, isCSTRSSymb)
import MuTerm.Framework.Problem (Problem (..), MkProblem, mkProblem)
import MuTerm.Framework.Processor (Processor (..))
import MuTerm.Framework.Proof (dontKnow, success, singleP, Proof, Info (..))

import Text.PrettyPrint.HughesPJClass as Doc (Pretty (..), text, (<+>), (<>))
import Control.Monad (MonadPlus)
import Data.Set as S (Set, elems)

-----------------------------------------------------------------------------
-- Data
-----------------------------------------------------------------------------


data NotCSTRSProcessor  = NotCSTRSProcessor

data NotCSTRSProcInfo problem = 
    NotCSTRSProcInfo { inProblem :: problem }
  | NotCSTRSProcDontKnow { inProblem :: problem }

-----------------------------------------------------------------------------
-- Instances
-----------------------------------------------------------------------------

-- Processor

instance (
  conftrs ~ InConfCSTRS trs
  , IsConfProblem conftrs trs
  , HasSignature trs idF idV
  , HasRules trs (Term idF idV)
  , HasArity idF
  , HasMu idF
  , Eq idF, Eq idV
  , Ord idF, Ord idV
  , Pretty idF, Pretty idV, Pretty trs
  , Pretty (Problem Rewriting conftrs)
  , Info info (NotCSTRSProcInfo (Problem Rewriting conftrs))
  ) => Processor info NotCSTRSProcessor (Problem Rewriting conftrs) (Problem Rewriting conftrs) where
  apply NotCSTRSProcessor inP
    | cstrs = dontKnow (NotCSTRSProcDontKnow inP) inP
    | otherwise = singleP (NotCSTRSProcInfo inP) inP inP
    where trs = getConfProblem . getR $ inP
          cstrs = or . map isCSTRSSymb . elems $ getTRSSymbols trs

-- Pretty

instance (
  IsConfProblem conftrs trs
  , HasSignature trs idF idV
  , HasRules trs (Term idF idV)
  , Pretty idF, Pretty idV, Pretty trs
  , Pretty (Problem Rewriting conftrs)
  ) => Pretty (NotCSTRSProcInfo (Problem Rewriting conftrs)) where
  pPrint NotCSTRSProcInfo { inProblem    = problem}
    = text "\nNot CS-TRS Processor:\n"
      Doc.<> text "R is not a CS-TRS"

-----------------------------------------------------------------------------
-- Functions
-----------------------------------------------------------------------------

-- | Applies the Not CS-TRS Processor
notcstrsProc :: (Processor info NotCSTRSProcessor o d
          , MonadPlus mp
          , Info info o
          , Info info d) => o -> Proof info mp d
notcstrsProc  = apply NotCSTRSProcessor
