{-# LANGUAGE FlexibleContexts #-}
{-# LANGUAGE TypeFamilies #-}
{-# LANGUAGE NoMonomorphismRestriction #-}
-----------------------------------------------------------------------------
-- |
-- Module      :  Framework.Strategies
-- Copyright   :  (c) muterm development team
-- License     :  see LICENSE
--
-- Maintainer  :  rgutierrez@dsic.upv.es
-- Stability   :  unstable
-- Portability :  non-portable
--
-- This module manage the different strategies to solve a confluence
-- problem
--
-----------------------------------------------------------------------------

module Framework.Strategies (

-- * Exported functions

rewStrat, crewStrat

) where

import MuTerm.Framework.Processor(Processor(..))
import Framework.Processor.IdProcessor (idProc)
import Framework.Processor.Rewriting.HuetInfProcessor (huetIProc)
import Framework.Processor.Rewriting.HuetBFJProcessor (huetBFJProc)
import Framework.Processor.Rewriting.HuetLevyProcessor (huetLevyProc)
import Framework.Processor.Rewriting.HuetLevyNWProcessor (huetLevyNWProc)
import Framework.Processor.Rewriting.TerminationProcessor (termProc)
import Framework.Processor.Rewriting.LinearityProcessor (linProc)
import Framework.Processor.Rewriting.SCRProcessor (scrProc)
import Framework.Processor.Rewriting.NoConvBFProcessor (ncbfProc)
import Framework.Processor.Rewriting.NoConvInfProcessor (ncIProc)
import Framework.Processor.Rewriting.CleanTRSProcessor (cleanTRSProc)
import Framework.Processor.Rewriting.DifferentNormalCPTermsProcessor (dncptProc)
import Framework.Processor.Rewriting.GroundNormalCPInfProcessor (gncpiProc)
import Framework.Processor.Rewriting.SCRInfProcessor (scrIProc)
import Framework.Processor.Rewriting.ModularConfCombsTRSProcessor (mccTRSProc)
import Framework.Processor.Rewriting.LHRVProcessor (lhrvProc)
import Framework.Processor.Rewriting.NotCSTRSProcessor (notcstrsProc)
import Framework.Processor.Rewriting.CSTRSProcessor (cstrsProc)
import Framework.Processor.Rewriting.ECPsNoLHRVProcessor (ecpnlhrvProc)
import Framework.Processor.Rewriting.LHCPsProcessor (lhcpsProc)
import Framework.Processor.Rewriting.Prov9LHCPsConvProcessor (prov9LHCPCvProc)
import Framework.Processor.Rewriting.CSRConverterFromCanonicalProcessor (csrcfCanProc)
import Framework.Processor.Rewriting.CSRConverterFromTopProcessor (csrcfTopProc)
import Framework.Processor.Rewriting.WNormalizationFortProcessor (wnFortProc)
import Framework.Processor.CRewriting.CriticalPairsCProcessor (critPairsCProc)
import Framework.Processor.CRewriting.OpTerminationCProcessor (opTermCProc)
import Framework.Processor.CRewriting.UnderlyingTRSTermCProcessor (undRTermCProc)
import Framework.Processor.CRewriting.TerminationCProcessor (termCProc)
import Framework.Processor.CRewriting.CleanCTRSProcessor (cleanCTRSProc)
import Framework.Processor.CRewriting.InliningOfCondsProcessor (inlCondsProc)
import Framework.Processor.CRewriting.CleanInfCCPsCProcessor (cleanICCPs)
import Framework.Processor.CRewriting.Prov9ConvCProcessor (prov9CvCProc)
import Framework.Processor.CRewriting.DiffGroundAndNormalCCPTermsProcessor (dganCCPProc)
import Framework.Processor.CRewriting.DiffGroundAndNormalNoCdsCCPTermsProcessor (dganncdsCCPProc)
import Framework.Processor.CRewriting.InfConvNoCdsCCPProcessor (infConvNCdsCCPProc)
import Framework.Processor.CRewriting.InfConvCCPProcessor (infConvCCPProc)
import Framework.Processor.CRewriting.InfConvConstCCPProcessor (infConvConstCCPProc)
import Framework.Processor.CRewriting.NormalRCondsCProcessor (normRCondsCProc)
import Framework.Processor.CRewriting.RightStableCProcessor (rightStbCProc)
import Framework.Processor.CRewriting.UTransformCTRSProcessor (uTransformCTRSProc)
import Framework.Processor.CRewriting.UOptTransformCTRSProcessor (uOptTransformCTRSProc)
import Framework.Processor.CRewriting.NoCondsTransformCTRS (noCondsTrCTRSProc)
import Framework.Processor.CRewriting.CCPsDistrCProcessor (ccPairsDistrProc)
import Framework.Processor.CRewriting.ConvOverlayCCPProcessor (convOvProc)
import MuTerm.Framework.Strategy(
    (.|.), (.||.), (.|||.),
    (.&.), final
    )
import Constraint.Interpretations (RelationType (..), Relation (..)
  , Interpretation(..), InterpType (..), InterpDomain (..))
import Solver.SolverSMTExt(Solver (..))

-----------------------------------------------------------------------------
-- Functions
-----------------------------------------------------------------------------

{-- Possible strategies

rewStrat timeout
    = idProc .&. 
    cleanTRSProc .&.
    mccTRSProc .&.
    (
    lhrvProc .&.
        (linProc .&. huetLevyProc .&. (scrProc (Just 8) (Just 50)) .&. final)
        --(linProc .&. huetLevyProc .&. ((scrProc (Just 8) (Just 50)) .||. (scrIProc (Just 5))) .&. final)
    )
    .||.
        (       (ecpnlhrvProc .&. huetLevyProc .&. (dncptProc .||. (ncbfProc (Just 8) (Just 50)) .||. (gncpiProc (Just 10))) .&. final)
        .||.    (huetLevyProc .&. (dncptProc .||. (ncbfProc (Just 8) (Just 50)) .||. (gncpiProc (Just 5)) .||. (ncIProc (Just 30))) .&. final)
        .||.    (linProc .&. huetLevyProc .&. ((scrProc (Just 8) (Just 50)) .||. (scrIProc (Just 5))) .&. final)
        .||.    (csrcfCanProc .&. 
                    (
                            ((wnFortProc (Just 5)) .&. huetLevyProc .&. ((huetBFJProc (Just 10) (Just 50)) .||. (huetIProc (Just 30))) .&. final)
                    .||.    ((termProc (Just 30)) .&. huetLevyProc .&. ((huetBFJProc (Just 10) (Just 50)) .||. (huetIProc (Just 30))) .&. final)
                    )
                )
        .||.    (csrcfTopProc .&.
                    (
                            ((wnFortProc (Just 5)) .&. huetLevyProc .&. ((huetBFJProc (Just 10) (Just 50)) .||. (huetIProc (Just 30))) .&. final)
                    .||.    ((termProc (Just 30)) .&. huetLevyProc .&. ((huetBFJProc (Just 10) (Just 50)) .||. (huetIProc (Just 30))) .&. final)
                    )
                )
        .||.    ((termProc (Just 30)) .&. huetLevyProc .&. ((huetBFJProc (Just 10) (Just 50)) .||. (huetIProc (Just 30))) .&. final)
        )

--}

--
rewStrat timeout
    = idProc .&. 
    cleanTRSProc .&.
    mccTRSProc .&.
    (
    lhrvProc .&.
        (
                (huetLevyProc .&. (dncptProc .||. (ncbfProc (Just 8) (Just 50))) .&. final)
        .|.    ((termProc (Just 30)) .&. huetLevyNWProc .&. ((huetBFJProc (Just 10) (Just 50)) .||. (huetIProc (Just 30))) .&. final)
        )
    )
    .||.
        (       
            (cstrsProc .&.
                (
                    (huetLevyProc .&. (dncptProc .||. (ncbfProc (Just 8) (Just 50))) .&. final)
                .||.    
                    --(huetLevyProc .&.
                    --(ecpnlhrvProc .&. huetLevyProc .&. 
                    (lhcpsProc .&. huetLevyProc .&. 
                        (     dncptProc 
                        .||. (ncbfProc (Just 8) (Just 50)) 
                        .||. (gncpiProc (Just 15)) 
                        .||. (ncIProc (Just 30))
                        ) .&. final
                    )
                .||.
                    --((termProc (Just 30)) .&. huetLevyNWProc .&. 
                    ((termProc (Just 30)) .&. lhcpsProc .&. huetLevyNWProc .&. 
                        (   ((huetBFJProc (Just 10) (Just 50)) .&. prov9LHCPCvProc (Just 30)) 
                        .||. (huetIProc (Just 30))
                        ) .&. final)
                )
            )
        .||.    
            (notcstrsProc .&. 
                (       
                        (linProc .&. huetLevyProc .&. (scrProc (Just 8) (Just 50)) .&. final)
                .||.    (
                            (huetLevyProc .&. dncptProc .&. final)
                        .|. (huetLevyProc .&. ((ncbfProc (Just 8) (Just 50)) .||. (gncpiProc (Just 15)) .||. (ncIProc (Just 30))) .&. final)
                        )
                .||.    (csrcfCanProc .&. 
                            (
                                    ((wnFortProc (Just 5)) .&. huetLevyNWProc .&. ((huetBFJProc (Just 10) (Just 50)) .||. (huetIProc (Just 30))) .&. final)
                            .||.    ((termProc (Just 30)) .&. huetLevyNWProc .&. ((huetBFJProc (Just 10) (Just 50)) .||. (huetIProc (Just 30))) .&. final)
                            )
                        )
                .||.    (csrcfTopProc .&.
                            (
                                    ((wnFortProc (Just 5)) .&. huetLevyNWProc .&. ((huetBFJProc (Just 10) (Just 50)) .||. (huetIProc (Just 30))) .&. final)
                            .||.    ((termProc (Just 30)) .&. huetLevyNWProc .&. ((huetBFJProc (Just 10) (Just 50)) .||. (huetIProc (Just 30))) .&. final)
                            )
                        )
                .||.    ((termProc (Just 30)) .&. huetLevyNWProc .&. ((huetBFJProc (Just 10) (Just 50)) .||. (huetIProc (Just 30))) .&. final)
                )
            )
        )
--}


crewStrat (Just timeout)
  =
    idProc .&.
    inlCondsProc .&.
    (
        (critPairsCProc .&.
        (
            ((rightStbCProc (Just 5)) .&.
             (normRCondsCProc (Just 5)) .&.
              final)
        .||.
            (uOptTransformCTRSProc .&.
             linProc .&. 
             huetLevyProc .&. 
            (scrProc (Just 8) (Just 50)) .&. 
             final)
        .||.
            (uTransformCTRSProc .&.
             linProc .&. 
             huetLevyProc .&. 
            (scrProc (Just 8) (Just 50)) .&. 
             final)
        .||.
            ((undRTermCProc (Just 5)) .&.
            (rightStbCProc (Just 5)) .&.
            (normRCondsCProc (Just 5)) .&.
             final)
        ) .&. final)
    .||.
        (cleanCTRSProc (Just (div timeout 2))) .&.
        (
            (noCondsTrCTRSProc .&.
             cleanTRSProc .&.
             mccTRSProc .&.
            (       
                        (linProc .&. huetLevyProc .&. (scrProc (Just 8) (Just 50)) .&. final)
                .||.    (
                            (huetLevyProc .&. dncptProc .&. final)
                        .|. (huetLevyProc .&. ((ncbfProc (Just 8) (Just 50)) .||. (gncpiProc (Just 15)) .||. (ncIProc (Just 30))) .&. final)
                        )
                .||.    (csrcfCanProc .&. 
                            (
                                    ((wnFortProc (Just 5)) .&. huetLevyNWProc .&. ((huetBFJProc (Just 10) (Just 50)) .||. (huetIProc (Just 30))) .&. final)
                            .||.    ((termProc (Just 30)) .&. huetLevyNWProc .&. ((huetBFJProc (Just 10) (Just 50)) .||. (huetIProc (Just 30))) .&. final)
                            )
                        )
                .||.    (csrcfTopProc .&.
                            (
                                    ((wnFortProc (Just 5)) .&. huetLevyNWProc .&. ((huetBFJProc (Just 10) (Just 50)) .||. (huetIProc (Just 30))) .&. final)
                            .||.    ((termProc (Just 30)) .&. huetLevyNWProc .&. ((huetBFJProc (Just 10) (Just 50)) .||. (huetIProc (Just 30))) .&. final)
                            )
                        )
                .||.    ((termProc (Just 30)) .&. huetLevyNWProc .&. ((huetBFJProc (Just 10) (Just 50)) .||. (huetIProc (Just 30))) .&. final)
                ) .&. final)
        .||.
            critPairsCProc .&.
            (
                ((rightStbCProc (Just 5)) .&.
                (normRCondsCProc (Just 5)) .&.
                final
                )
            .||. 
                (ccPairsDistrProc .&.
                (
                    (dganncdsCCPProc (Just (div timeout 3)))
                .||.
                    (infConvNCdsCCPProc (Just (div timeout 2)))
                ) .&.
                final)
            .||. 
                (ccPairsDistrProc .&.
                (infConvConstCCPProc (Just (div timeout 2))) .&.
                final)
            .||.
                ((cleanICCPs (Just (div timeout 2))) .&.
                (
                    (ccPairsDistrProc .&.
                    (
                        (dganCCPProc (Just (div timeout 3)))
                    .||.
                        (infConvCCPProc (Just (div timeout 2)))
                    ) .&.
                    final)
                .||.
                    ((undRTermCProc (Just (div timeout 2))) .&.
                    (
                        ((rightStbCProc (Just (div timeout 3))) .&.
                        (normRCondsCProc (Just (div timeout 3))) .&.
                        (
                            (ccPairsDistrProc .&.
                            (prov9CvCProc (Just (div timeout 2))) .&.
                            final)
                        .||.
                            ((convOvProc (Just (div timeout 2))) .&.
                            final)
                        ) .&. 
                        final)
                    ) .&. final
                    )
                .||.
                    ((opTermCProc (Just (div timeout 2))) .&.
                    (
                        ((rightStbCProc (Just (div timeout 3))) .&. 
                        (normRCondsCProc (Just (div timeout 3))) .&.
                        (
                            (ccPairsDistrProc .&.
                            (prov9CvCProc (Just (div timeout 2))) .&.
                            final)
                        .||.
                            ((convOvProc (Just (div timeout 2))) .&.
                            final)
                        ) .&.
                        final)
                    ) .&. final
                    )
                .||.
                    ((termCProc (Just (div timeout 2))) .&.
                    (
                        ((rightStbCProc (Just (div timeout 3))) .&.
                        (normRCondsCProc (Just (div timeout 3))) .&.
                        (
                            (ccPairsDistrProc .&.
                            (prov9CvCProc (Just (div timeout 2))) .&.
                            final)
                        .||.
                            ((convOvProc (Just (div timeout 2))) .&.
                            final)
                        ) .&. 
                        final)
                    ) .&. final
                    )
                ) .&.
                final)
            .|.
                (
                    (uOptTransformCTRSProc .&.
                        (
                            (linProc .&. huetLevyNWProc .&. (scrProc (Just 8) (Just 50)) .&. final)
                        .||.
                            (csrcfCanProc .&. 
                                (
                                        ((wnFortProc (Just (div timeout 6))) .&. huetLevyNWProc .&. (huetBFJProc (Just 8) (Just 50)) .&. final)
                                .||.    ((termProc (Just (div timeout 5))) .&. huetLevyNWProc .&. (huetBFJProc (Just 8) (Just 50)) .&. final)
                                )
                            )
                        .||.    
                            (csrcfTopProc .&.
                                (
                                        ((wnFortProc (Just (div timeout 6))) .&. huetLevyNWProc .&. (huetBFJProc (Just 8) (Just 50)) .&. final)
                                .||.    ((termProc (Just (div timeout 5))) .&. huetLevyNWProc .&. (huetBFJProc (Just 8) (Just 50)) .&. final)
                                )
                            )
                        .||.
                            ((termProc (Just (div timeout 5))) .&. huetLevyNWProc .&. (huetBFJProc (Just 8) (Just 50)) .&. final)
                        ) .&.
                        final)
                .||.
                    (uTransformCTRSProc .&.
                        (
                            (linProc .&. huetLevyNWProc .&. (scrProc (Just 8) (Just 50)) .&. final)
                        .||.
                            (csrcfCanProc .&. 
                                (
                                        ((wnFortProc (Just (div timeout 6))) .&. huetLevyNWProc .&. (huetBFJProc (Just 8) (Just 50)) .&. final)
                                .||.    ((termProc (Just (div timeout 5))) .&. huetLevyNWProc .&. (huetBFJProc (Just 8) (Just 50)) .&. final)
                                )
                            )
                        .||.    
                            (csrcfTopProc .&.
                                (
                                        ((wnFortProc (Just (div timeout 6))) .&. huetLevyNWProc .&. (huetBFJProc (Just 8) (Just 50)) .&. final)
                                .||.    ((termProc (Just (div timeout 5))) .&. huetLevyNWProc .&. (huetBFJProc (Just 8) (Just 50)) .&. final)
                                )
                            )
                        .||.
                            ((termProc (Just (div timeout 5))) .&. huetLevyNWProc .&. (huetBFJProc (Just 8) (Just 50)) .&. final)
                        ) .&.
                        final)
                )
            ) .&.
            final
        )
    )

{-
    (uOptTransformCTRSProc .&.
        (
            (linProc .&. huetLevyProc .&. (scrProc (Just 8) (Just 50)) .&. final)
        .||.
            (csrcfCanProc .&. 
                (
                        ((wnFortProc (Just 5)) .&. huetLevyProc .&. (huetBFJProc (Just 8) (Just 50)) .&. final)
                .||.    ((wnFortProc (Just (div timeout 6))) .&. huetLevyProc .&. ((huetBFJProc (Just 10) (Just 50)) .||. (huetIProc ((Just 30)))) .&. final)
                .||.    ((termProc (Just 5)) .&. huetLevyProc .&. (huetBFJProc (Just 8) (Just 50)) .&. final)
                .||.    ((termProc (Just (div timeout 5))) .&. huetLevyProc .&. ((huetBFJProc (Just 10) (Just 50)) .||. (huetIProc ((Just 30)))) .&. final)
                )
            )
        .||.    
            (csrcfTopProc .&.
                (
                        ((wnFortProc (Just 5)) .&. huetLevyProc .&. (huetBFJProc (Just 8) (Just 50)) .&. final)
                .||.    ((wnFortProc (Just (div timeout 6))) .&. huetLevyProc .&. ((huetBFJProc (Just 10) (Just 50)) .||. (huetIProc ((Just 30)))) .&. final)
                .||.    ((termProc (Just 5)) .&. huetLevyProc .&. (huetBFJProc (Just 8) (Just 50)) .&. final)
                .||.    ((termProc (Just (div timeout 5))) .&. huetLevyProc .&. ((huetBFJProc (Just 10) (Just 50)) .||. (huetIProc ((Just 30)))) .&. final)
                )
            )
        .||.    
            ((termProc (Just 5)) .&. huetLevyProc .&. (huetBFJProc (Just 8) (Just 50)) .&. final)
        .||.
            ((termProc (Just (div timeout 3))) .&. huetLevyProc .&. ((huetBFJProc (Just 10) (Just 50)) .||. (huetIProc ((Just 30)))) .&. final)
        ) .&.
        final)
.||.
    (uTransformCTRSProc .&.
        (
            (linProc .&. huetLevyProc .&. (scrProc (Just 8) (Just 50)) .&. final)
        .||.
            (csrcfCanProc .&. 
                (
                        ((wnFortProc (Just 5)) .&. huetLevyProc .&. (huetBFJProc (Just 8) (Just 50)) .&. final)
                .||.    ((wnFortProc (Just (div timeout 6))) .&. huetLevyProc .&. ((huetBFJProc (Just 10) (Just 50)) .||. (huetIProc ((Just 30)))) .&. final)
                .||.    ((termProc (Just 5)) .&. huetLevyProc .&. (huetBFJProc (Just 8) (Just 50)) .&. final)
                .||.    ((termProc (Just (div timeout 5))) .&. huetLevyProc .&. ((huetBFJProc (Just 10) (Just 50)) .||. (huetIProc ((Just 30)))) .&. final)
                )
            )
        .||.    
            (csrcfTopProc .&.
                (
                        ((wnFortProc (Just 5)) .&. huetLevyProc .&. (huetBFJProc (Just 8) (Just 50)) .&. final)
                .||.    ((wnFortProc (Just (div timeout 6))) .&. huetLevyProc .&. ((huetBFJProc (Just 10) (Just 50)) .||. (huetIProc ((Just 30)))) .&. final)
                .||.    ((termProc (Just 5)) .&. huetLevyProc .&. (huetBFJProc (Just 8) (Just 50)) .&. final)
                .||.    ((termProc (Just (div timeout 5))) .&. huetLevyProc .&. ((huetBFJProc (Just 10) (Just 50)) .||. (huetIProc ((Just 30)))) .&. final)
                )
            )
        .||.    
            ((termProc (Just 5)) .&. huetLevyProc .&. (huetBFJProc (Just 8) (Just 50)) .&. final)
        .||.
            ((termProc (Just (div timeout 3))) .&. huetLevyProc .&. ((huetBFJProc (Just 10) (Just 50)) .||. (huetIProc ((Just 30)))) .&. final)
        ) .&.
        final)
-}


{-
crewStrat (Just timeout)
  =  
    idProc .&.
    inlCondsProc .&.
    (
        (critPairsCProc .&.
        (
            ((rightStbCProc (Just 5)) .&.
             (normRCondsCProc (Just 5)) .&.
              final)
        .||.
            ((undRTermCProc (Just 5)) .&.
            (
                ((rightStbCProc (Just 5)) .&.
                 (normRCondsCProc (Just 5)) .&.
                  final)
            .||. 
                (uOptTransformCTRSProc .&. 
                 huetLevyProc .&. 
                (huetBFJProc (Just 8) (Just 50)) .&.
                 final)
            ) .&. final
            )
        ) .&. final)
    .||.
        (cleanCTRSProc (Just (div timeout 2))) .&.
        (
            (noCondsTrCTRSProc .&.
             cleanTRSProc .&.
             mccTRSProc .&.
            (       
                        (linProc .&. huetLevyProc .&. (scrProc (Just 8) (Just 50)) .&. final)
                .||.    (csrcfCanProc .&. (wnFortProc (Just 5)) .&. huetLevyProc .&. ((huetBFJProc (Just 8) (Just 50)) .||. (huetIProc (Just 10))) .&. final)
                .||.    (csrcfTopProc .&. (wnFortProc (Just 5)) .&. huetLevyProc .&. ((huetBFJProc (Just 8) (Just 50)) .||. (huetIProc (Just 10))) .&. final)
                .||.    (huetLevyProc .&. (dncptProc .||. (ncbfProc (Just 8) (Just 50)) .||. (gncpiProc (Just 15)) .||. (ncIProc (Just 30))) .&. final)
                .||.    (csrcfCanProc .&. 
                            (
                                    ((wnFortProc (Just 5)) .&. huetLevyProc .&. ((huetBFJProc (Just 10) (Just 50)) .||. (huetIProc (Just 30))) .&. final)
                            .||.    ((termProc (Just 30)) .&. huetLevyProc .&. ((huetBFJProc (Just 10) (Just 50)) .||. (huetIProc (Just 30))) .&. final)
                            )
                        )
                .||.    (csrcfTopProc .&.
                            (
                                    ((wnFortProc (Just 5)) .&. huetLevyProc .&. ((huetBFJProc (Just 10) (Just 50)) .||. (huetIProc (Just 30))) .&. final)
                            .||.    ((termProc (Just 30)) .&. huetLevyProc .&. ((huetBFJProc (Just 10) (Just 50)) .||. (huetIProc (Just 30))) .&. final)
                            )
                        )
                .||.    ((termProc (Just 30)) .&. huetLevyProc .&. ((huetBFJProc (Just 10) (Just 50)) .||. (huetIProc (Just 30))) .&. final)
            ) .&. final)
        .||.
            critPairsCProc .&.
            (
                ((rightStbCProc (Just 5)) .&.
                (normRCondsCProc (Just 5)) .&.
                final
                )
            .||. 
                (ccPairsDistrProc .&.
                (
                    (dganncdsCCPProc (Just (div timeout 3)))
                .||.
                    (infConvNCdsCCPProc (Just (div timeout 2)))
                ) .&.
                final)
            .||. 
                (ccPairsDistrProc .&.
                (infConvConstCCPProc (Just (div timeout 2))) .&.
                final)
            .||.
                (cleanICCPs (Just (div timeout 2))) .&.
                (
                    (ccPairsDistrProc .&.
                    (
                        (dganCCPProc (Just (div timeout 3)))
                    .||.
                        (infConvCCPProc (Just (div timeout 2)))
                    ) .&.
                    final)
                .||.
                    ((undRTermCProc (Just 5)) .&.
                    (
                        ((rightStbCProc (Just 5)) .&.
                        (normRCondsCProc (Just 5)) .&.
                        (
                            (ccPairsDistrProc .&.
                            (prov9CvCProc (Just 5)) .&.
                            final)
                        .||.
                            ((convOvProc (Just 5)) .&.
                            final)
                        ) .&. 
                        final)
                    .||. 
                        (uOptTransformCTRSProc .&.
                        (
                            (huetLevyProc .&. (huetBFJProc (Just 8) (Just 50)) .&. final)
                        .||.
                            (csrcfCanProc .&. huetLevyProc .&. ((huetBFJProc (Just 8) (Just 50)) .||. (huetIProc (Just 3))) .&. final)
                        .||.
                            (csrcfTopProc .&. huetLevyProc .&. ((huetBFJProc (Just 8) (Just 50)) .||. (huetIProc (Just 3))) .&. final) 
                        ) .&.
                        final)
                    ) .&. final
                    )
                .||.
                    ((opTermCProc (Just 5)) .&.
                    (
                        ((rightStbCProc (Just 5)) .&. 
                        (normRCondsCProc (Just 5)) .&.
                        (
                            (ccPairsDistrProc .&.
                            (prov9CvCProc (Just 5)) .&.
                            final)
                        .||.
                            ((convOvProc (Just 5)) .&.
                            final)
                        ) .&.
                        final)
                    .||. 
                        (uOptTransformCTRSProc .&.
                        (
                            (huetLevyProc .&. (huetBFJProc (Just 8) (Just 50)) .&. final)
                        .||.
                            (csrcfCanProc .&. huetLevyProc .&. ((huetBFJProc (Just 8) (Just 50)) .||. (huetIProc (Just 3))) .&. final)
                        .||.
                            (csrcfTopProc .&. huetLevyProc .&. ((huetBFJProc (Just 8) (Just 50)) .||. (huetIProc (Just 3))) .&. final) 
                        ) .&.
                        final)
                    ) .&. final
                    )
                .||.
                    ((termCProc (Just 5)) .&.
                    (
                        ((rightStbCProc (Just 5)) .&.
                        (normRCondsCProc (Just 5)) .&.
                        (
                            (ccPairsDistrProc .&.
                            (prov9CvCProc (Just 5)) .&.
                            final)
                        .||.
                            ((convOvProc (Just 5)) .&.
                            final)
                        ) .&. 
                        final)
                    .||. 
                        (uOptTransformCTRSProc .&.
                        (
                            (huetLevyProc .&. (huetBFJProc (Just 8) (Just 50)) .&. final)
                        .||.
                            (csrcfCanProc .&. huetLevyProc .&. ((huetBFJProc (Just 8) (Just 50)) .||. (huetIProc (Just 3))) .&. final)
                        .||.
                            (csrcfTopProc .&. huetLevyProc .&. ((huetBFJProc (Just 8) (Just 50)) .||. (huetIProc (Just 3))) .&. final) 
                        ) .&.
                        final)
                    ) .&. final
                    )
                .||.
                    ((undRTermCProc (Just (div timeout 2))) .&.
                    (
                        ((rightStbCProc (Just (div timeout 5))) .&.
                        (normRCondsCProc (Just (div timeout 5))) .&.
                        (
                            (ccPairsDistrProc .&.
                            (prov9CvCProc (Just (div timeout 2))) .&.
                            final)
                        .||.
                            ((convOvProc (Just (div timeout 2))) .&.
                            final)
                        ) .&. 
                        final)
                    .||.
                        (uOptTransformCTRSProc .&.
                        (
                            (huetLevyProc .&. ((huetBFJProc (Just 8) (Just 50)) .||. (huetIProc (Just ((div timeout 3))))) .&. final)
                        .||.
                            (csrcfCanProc .&. huetLevyProc .&. ((huetBFJProc (Just 8) (Just 50)) .||. (huetIProc (Just (div timeout 3)))) .&. final)
                        .||.
                            (csrcfTopProc .&. huetLevyProc .&. ((huetBFJProc (Just 8) (Just 50)) .||. (huetIProc (Just (div timeout 3)))) .&. final) 
                        ) .&.
                        final)
                    ) .&. final
                    )
                .||.
                    ((opTermCProc (Just (div timeout 2))) .&.
                    (
                        ((rightStbCProc (Just (div timeout 5))) .&. 
                        (normRCondsCProc (Just (div timeout 5))) .&.
                        (
                            (ccPairsDistrProc .&.
                            (prov9CvCProc (Just (div timeout 2))) .&.
                            final)
                        .||.
                            ((convOvProc (Just (div timeout 2))) .&.
                            final)
                        ) .&.
                        final)
                    .||. 
                        (uOptTransformCTRSProc .&.
                        (
                            (huetLevyProc .&. ((huetBFJProc (Just 8) (Just 50)) .||. (huetIProc (Just (div timeout 3)))) .&. final)
                        .||.
                            (csrcfCanProc .&. huetLevyProc .&. ((huetBFJProc (Just 8) (Just 50)) .||. (huetIProc (Just (div timeout 3)))) .&. final)
                        .||.
                            (csrcfTopProc .&. huetLevyProc .&. ((huetBFJProc (Just 8) (Just 50)) .||. (huetIProc (Just (div timeout 3)))) .&. final) 
                        ) .&.
                        final)
                    ) .&. final
                    )
                .||.
                    ((termCProc (Just (div timeout 2))) .&.
                    (
                        ((rightStbCProc (Just (div timeout 5))) .&.
                        (normRCondsCProc (Just (div timeout 5))) .&.
                        (
                            (ccPairsDistrProc .&.
                            (prov9CvCProc (Just (div timeout 2))) .&.
                            final)
                        .||.
                            ((convOvProc (Just (div timeout 2))) .&.
                            final)
                        ) .&. 
                        final)
                    .||.
                        (uOptTransformCTRSProc .&.
                        (
                            (huetLevyProc .&. ((huetBFJProc (Just 8) (Just 50)) .||. (huetIProc (Just (div timeout 3)))) .&. final)
                        .||.
                            (csrcfCanProc .&. huetLevyProc .&. ((huetBFJProc (Just 8) (Just 50)) .||. (huetIProc (Just (div timeout 3)))) .&. final)
                        .||.
                            (csrcfTopProc .&. huetLevyProc .&. ((huetBFJProc (Just 8) (Just 50)) .||. (huetIProc (Just (div timeout 3)))) .&. final) 
                        ) .&.
                        final)
                    ) .&. final
                    )
                ) .&.
                final
            ) .&.
            final
        )
    )
-}
