{-# LANGUAGE DeriveDataTypeable #-}
{-# LANGUAGE FlexibleContexts #-}
--------------------------------------------------------------------------
-- |
-- Module      :  Main
-- Copyright   :  (c) muterm development team
-- License     :  see LICENSE
--
-- Maintainer  :  rgutierrez@dsic.upv.es
-- Stability   :  unstable
-- Portability :  non-portable
--
-- This is the inProver main module.
--
-----------------------------------------------------------------------------

module Main (

-- * Exported functions

main

) where

import Interface.CLI (Opt (..), parseOptions, autoparse)
import Framework.Problem.Types(MProblem (..))
import MuTerm.Framework.Problem (IsProblem (..))
import MuTerm.Framework.Proof (someInfo)
import Framework.Proofs (runProofSol)
import Framework.Strategies (rewStrat, crewStrat)

import Text.PrettyPrint.HughesPJClass (Pretty (..))

-----------------------------------------------------------------------------
-- Functions
-----------------------------------------------------------------------------

-- | The 'main' function parses an input file and returns a answer
-- about the feasability/infeasibility of the input problem.
main :: IO ()
main =
  do (opts, _) <- parseOptions
     let Opt { optName = name
             , optInput = input
             , optOutput = output
             , optFormat = format
             , optTimeout = timeout'
             } = opts
     let timeout = case timeout' of
                     Nothing -> Nothing
                     Just to -> Just to
                     --Just to -> Just $ div to 3
     filedata <- input
     let trs =  autoparse name filedata
     case trs of
       MRewriting problem -> do {
         let proof = rewStrat timeout problem
         ; let sol = runProofSol proof
         ; output format (someInfo problem) sol
         }
       MCRewriting problem -> do {
         let proof = crewStrat timeout problem
         ; let sol = runProofSol proof
         ; output format (someInfo problem) sol
         }
---       _ -> error "Error (Main): Input problem not supported"
