#!/bin/bash

if [ $# != 1 ]; then 
  echo "A proofs directory is needed"
  echo "./processors_analysis.sh [proofs-directory]"
  return 1
fi

if [ ! -d $1 ]; then
  echo "$1 directory not found"
  echo "./processors_analysis.sh [proofs-directory]"
  return 1
else
  cd $1
fi

declare -A processors
processors["Conditional Critical Pairs Distributor Processor"]=0
processors["Clean CTRS Processor"]=0
processors["Clean Infeasible CCPs Processor"]=0
processors["Convergent Overlay CCP Processor"]=0
processors["Critical Pairs Processor"]=0
processors["InfChecker Different Ground and Normal CCP Terms Processor"]=0
processors["InfChecker Different Ground and Normal No Conditions CCP Terms Processor"]=0
processors["Infeasible Convergence CCP Processor"]=0
processors["Infeasible Convergence No Conditions CCP Processor"]=0
processors["Inlining of Conditions Processor"]=0
processors["Normal RConds Processor"]=0
processors["Operation Termination Processor"]=0
processors["Prover9 CCP Convergence Checker Processor"]=0
processors["Right Stable Processor"]=0
processors["Termination CProcessor"]=0
processors["Underlying TRS Termination Processor"]=0
processors["Transform CTRS Processor"]=0
processors["CleanTRS Processor"]=0
processors["CSR Converter From Canonical u-Replacement Map Processor"]=0
processors["CSR Converter From Top u-Replacement Map Processor"]=0
processors["Different Normal CP Terms Processor"]=0
processors["Extended u-Critical Pairs NonLHRV Processor"]=0
processors["Ground Normal CP InfChecker Processor"]=0
processors["Huet Brute Force Joinability Processor"]=0
processors["Huet InfChecker Processor"]=0
processors["Huet Levy Processor"]=0
processors["Left-Homogeneous u-Replacing Variables Processor"]=0
processors["Linearity Processor"]=0
processors["Modular Confluence Combinations Decomposition Processor"]=0
processors["No Convergence Brute Force Processor"]=0
processors["No Convergence InfChecker Processor"]=0
processors["Strong Confluence InfChecker Processor"]=0
processors["Strong Confluence Processor"]=0
processors["Termination Processor"]=0
processors["Weak Normalization FORT Processor"]=0
declare -A exWhichUsedProc
for k in "${!processors[@]}"; do
  exWhichUsedProc[$k]=0
done

#echo ${processors[@]}
#for k in "${!processors[@]}"; do
#  echo "${k}: ${processors[$k]}"
#  procs=$(cat "404.trs/out.txt" | grep "$k" | wc -l)
#done

files=$(ls | sort -n)
nfiles=${#files[@]}
totalProcsReps=0
totalProcsUsed=0
for f in $files
do
  file="$f/out.txt"
  echo "------------------------------------------------------------------"
  echo $file
  #sudo chmod 555 $file
  for k in "${!processors[@]}"; do
    procs=$(cat $file | grep "$k" | wc -l)
  	echo "  -> $k: $procs"
  	processors[$k]=$((procs + processors[$k]))
  	totalProcsReps=$((totalProcsReps + $procs))
  	if [[ procs -gt 0 ]]; then
  	  totalProcsUsed=$((1 + totalProcsUsed))
  	  exWhichUsedProc[$k]=$((1 + exWhichUsedProc[$k]))
  	fi
  done
	echo "------------------------------------------------------------------"
	echo ""
done
echo "------------------------------------------------------------------"
echo "----------------------C O N C L U S I O N S-----------------------"
echo "------------------------------------------------------------------"
echo ""
totperc=`echo 100/$totalProcsReps | bc -l`
eachperc=`echo 100/$totalProcsUsed | bc -l`
for k in "${!processors[@]}"; do
  echo "-> $k"
  echo "    Total: ${processors[$k]} (`echo ${processors[$k]}*$totperc | bc -l`%)"
  echo "    Examples which used it: ${exWhichUsedProc[$k]} (`echo ${exWhichUsedProc[$k]}*$eachperc | bc -l`%)"
  echo ""
done
echo ""
echo "Total CONFident Processors: $totalProcsReps"
echo "Different Processors used for each example: $totalProcsUsed"
echo "Total FILES analysed: $nfiles"


# Aux function
calc(){ awk "BEGIN { print "$*" }"; }
